Package: simcross
Version: 0.10
Date: 2026-06-08
Title: Simulate Experimental Crosses
Description: Simulate and plot general experimental crosses. The focus is on simulating genotypes with an aim towards flexibility rather than speed. Meiosis is simulated following the Stahl model, in which chiasma locations are the superposition of two processes: a proportion p coming from a process exhibiting no interference, and the remainder coming from a process following the chi-square model.
Author: Karl W Broman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4914-6671>)
Maintainer: Karl W Broman <broman@wisc.edu>
Authors@R: c(person("Karl W", "Broman", role=c("aut", "cre"),
             email="broman@wisc.edu", comment=c(ORCID = "0000-0002-4914-6671")))
Depends: R (>= 3.1.0)
Imports: graphics, stats, Rcpp (>= 0.12.17)
Suggests: qtl, knitr, rmarkdown, testthat, devtools, roxygen2
License: GPL-3
URL: https://kbroman.org/simcross/, https://github.com/kbroman/simcross
BugReports: https://github.com/kbroman/simcross/issues
VignetteBuilder: knitr
LinkingTo: Rcpp
LazyData: true
Encoding: UTF-8
ByteCompile: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-09 02:55:46 UTC; kbroman
Repository: CRAN
Date/Publication: 2026-06-09 03:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-09 06:46:41 UTC; unix
Archs: simcross.so.dSYM

# Generated by roxygen2: do not edit by hand

export(CCcolors)
export(calc_Lstar)
export(check_pedigree)
export(collapse_do_alleles)
export(convert2geno)
export(convert2geno_allchr)
export(create_parent)
export(cross)
export(get_geno)
export(plot_crosslines)
export(plot_ind)
export(sim_4way_pedigree)
export(sim_ail_pedigree)
export(sim_ail_pedigree_fix_n)
export(sim_crossovers)
export(sim_do_pedigree)
export(sim_do_pedigree_fix_n)
export(sim_dof1_pedigree)
export(sim_from_pedigree)
export(sim_from_pedigree_allchr)
export(sim_meiosis)
export(sim_ril_pedigree)
export(where_het)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,rgb)
importFrom(graphics,arrows)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(stats,dpois)
importFrom(stats,uniroot)
useDynLib(simcross, .registration=TRUE)

## [R/simcross](https://kbroman.org/simcross/) <a href="https://kbroman.org/simcross/"><img src="https://kbroman.org/simcross/assets/pics/simcross_logo.png" align="right" height="138" alt="R/simcross logo"/></a>

[![R-CMD-check](https://github.com/kbroman/simcross/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kbroman/simcross/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/simcross)](https://cran.r-project.org/package=simcross)
[![r-universe badge](https://kbroman.r-universe.dev/simcross/badges/version)](https://kbroman.r-universe.dev/simcross)
[![zenodo DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.4032914.svg)](https://doi.org/10.5281/zenodo.4032914)

[Karl W Broman](https://kbroman.org)

---

R/simcross is an [R](https://www.r-project.org) package for simulating
and plotting general experimental crosses.

---

### Installation

You can install R/simcross from [CRAN](https://cran.r-project.org):

```r
install.packages("simcross")
```

Alternatively, install it from [R
universe](https://kbroman.r-universe.dev):

```r
install.packages("simcross", repos=c("https://kbroman.r-universe.dev",
                                     "https://cloud.r-project.org"))
```

Or use [remotes](https://remotes.r-lib.org) to install it from its GitHub source:

```r
install.packages("remotes")
remotes::install_github("kbroman/simcross")
```

---

### Vignette

A vignette describing the use of the package is available from within
R (and [also here](https://kbroman.org/simcross/assets/vignettes/simcross.html)). Load the package
and then use the `vignette` function.

```r
library(simcross)
vignette("simcross", package="simcross")
```

---

### License

This package is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License, version 3, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but
without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.  See the GNU
General Public License for more details.

A copy of the GNU General Public License, version 3, is available at
<https://www.r-project.org/Licenses/GPL-3>

## plot_cross.R

#  CCcolors
#
#' Collaborative Cross colors
#'
#' Get the vector of colors for the Collaborative Cross
#'
#' @param palette Which version of the colors to use? (New or original)
#'
#' @return vector of eight colors
#'
#' @keywords color
#' @export
#' @importFrom grDevices rgb
#' @importFrom graphics par points segments
#' @examples
#' CCcolors()
CCcolors <-
    function(palette=c("new", "original", "official")) {
        palette <- match.arg(palette)

        if(palette=="new") {
            return(
                c(AJ   ="#FFDC00",
                  B6   ="#888888",
                  "129"="#F08080",
                  NOD  ="#0064C9",
                  NZO  ="#7FDBFF",
                  CAST ="#2ECC40",
                  PWK  ="#FF4136",
                  WSB  ="#B10DC9")
            )
        }
        else {
            return(
                c("AJ"  =rgb(240,240,  0,maxColorValue=255),
                  "B6"  =rgb(128,128,128,maxColorValue=255),
                  "129" =rgb(240,128,128,maxColorValue=255),
                  "NOD" =rgb( 16, 16,240,maxColorValue=255),
                  "NZO" =rgb(  0,160,240,maxColorValue=255),
                  "CAST"=rgb(  0,160,  0,maxColorValue=255),
                  "PWK" =rgb(240,  0,  0,maxColorValue=255),
                  "WSB" =rgb(144,  0,224,maxColorValue=255))
            )
        }
    }

#  plot_ind
#
#' Plot an individual
#'
#' Add an individual, as a pair of chromosomes, to a plot
#'
#' @param ind An individual object, as output by
#' [create_parent()] or [cross()]
#' @param center (x,y) vector for the center of the individual
#' @param chrlength Length of chromosomes (Can be a vector of length
#' 2, in which case the two chromosomes will be different lengths,
#' aligned at the top. This is for the X chromosome.)
#' @param chrwidth Width of chromosomes
#' @param gap Gap between chromosomes
#' @param col Vector of colors
#' @param border Color for border
#' @param lend Passed to [graphics::rect()]
#' @param ljoin Passed to [graphics::rect()]
#' @param allborders If TRUE, put borders around all segments
#' @param ... Additional arguments passed to rect()
#'
#' @return None.
#'
#' @importFrom graphics rect
#' @keywords hplot
#' @export
#' @seealso [plot_crosslines()]
#' @examples
#' \dontshow{set.seed(67452378)}
#' mom <- create_parent(100, 1:2)
#' dad <- create_parent(100, 3:4)
#' kid <- cross(mom, dad)
#' plot(0,0, type="n", xlim=c(0, 100), ylim=c(0,100),
#'      xaxt="n", yaxt="n", xlab="", ylab="")
#' loc <- list(c(25,75), c(75,75), c(50,25))
#' plot_ind(mom, loc[[1]])
#' plot_ind(dad, loc[[2]])
#' plot_ind(kid, loc[[3]])
#' plot_crosslines(loc[[1]], loc[[2]], loc[[3]])
plot_ind <-
    function(ind, center, chrlength=30, chrwidth=3, gap=3, col=CCcolors(),
             border="black", lend=1, ljoin=1, allborders=FALSE, ...)
{
    max_alleles <- max(sapply(ind, function(a) max(a$alleles)))
    if(max_alleles > length(col))
        stop("Need more colors: length(col)=", length(col), " but max allele = ", max_alleles)

    if(length(chrlength)==0) stop("chrlength has length 0")
    if(length(chrlength)==1) chrlength <- rep(chrlength, 2)
    if(length(chrlength)>2) {
        warning("chrlength should have length 1 or 2; the first two values will be used")
        chrlength <- chrlength[1:2]
    }

    for(i in 1:2) {
        sgn <- i*2-3
        chr <- ind[[i]]
        pos <- c(0, ind[[i]]$locations)
        allele <- ind[[i]]$alleles

        # rescale pos
        top <- center[2]+max(chrlength)/2
        bottom <- top - chrlength[i]
        if(diff(par("usr")[3:4]) < 0) { # small values at top of figure
            tmp <- top
            top <- bottom
            bottom <- top
        }
        start <- pos[1]
        end <- pos[length(pos)]
        pos <- (pos-start)*(bottom-top)/(end-start) + top

        left <- center[1] + sgn*(gap/2+chrwidth)
        right <- center[1] + sgn*gap/2

        internalborder <- ifelse(allborders, border, NA)
        for(j in 2:length(pos))
            rect(rep(left, length(pos)-1),  pos[-length(pos)],
                 rep(right, length(pos)-1), pos[-1],
                 col=col[allele],
                 border=internalborder, lend=lend, ljoin=ljoin, ...)

        if(!is.na(border) && !is.null(border)) # draw border
            rect(center[1] + sgn*(gap/2+chrwidth), top,
                 center[1] + sgn*gap/2,         bottom,
                 col=NA, border=border, lend=lend, ljoin=ljoin, ...)
    }
    invisible(NULL)
}

#  plot_crosslines
#
#' Plot cross lines
#'
#' Add lines for a cross
#'
#' @param momloc An (x,y) vector with center location for mother
#' @param dadloc An (x,y) vector with center location for mother
#' @param kidsloc Either an (x,y) vector with center location for a
#' kid, or a list of such for multiple kids
#' @param gap Gap arrows and points/rectangles
#' @param cex Character expansion for x point
#' @param chrlength Length of chromosomes
#' @param lwd Line width for points, segments, and arrows
#' @param arrow_length The `length` parameter in the call to
#' [graphics::arrows()]
#' @param col Color of lines and points
#' @param ... Additional arguments passed to arrows() and segments()
#'
#' @return None.
#'
#' @importFrom graphics points arrows
#' @keywords hplot
#' @export
#' @seealso [plot_ind()]
#' @examples
#' \dontshow{set.seed(67452378)}
#' mom <- create_parent(100, 1:2)
#' dad <- create_parent(100, 3:4)
#' kids <- lapply(1:4, function(junk) cross(mom, dad))
#' plot(0,0, type="n", xlim=c(0, 100), ylim=c(0,100),
#'      xaxt="n", yaxt="n", xlab="", ylab="")
#' loc <- list(c(25,75), c(75,75), c(12.5,25), c(37.5,25), c(62.5, 25), c(87.5,25))
#' plot_ind(mom, loc[[1]])
#' plot_ind(dad, loc[[2]])
#' for(i in 1:4) plot_ind(kids[[i]], loc[[i+2]])
#' plot_crosslines(loc[[1]], loc[[2]], loc[3:6])
plot_crosslines <-
    function(momloc, dadloc, kidsloc, gap=3, chrlength=30, cex=1.5,
             lwd=2, arrow_length=0.1, col="black", ...)
{
    stopifnot(length(momloc)==2, length(dadloc)==2)

    point <- colMeans(rbind(momloc, dadloc))
    points(point[1], point[2], pch=4, cex=cex, lwd=2, col=col, ...)

    if(!is.list(kidsloc)) { # 1 kid
        stopifnot(length(kidsloc)==2)
        sgn <- sign(kidsloc[2] - point[2])
        arrows(point[1], point[2]+sgn*gap*2, kidsloc[1], kidsloc[2]-sgn*(chrlength/2+gap),
               lwd=lwd, length=arrow_length, col=col, ...)
    } else { # multiple kids
        if(any(vapply(kidsloc, length, 2) != 2))
            stop("kidsloc must be a list of vectors of length 2")

        kidx <- vapply(kidsloc, "[", 0, 1)
        kidy <- vapply(kidsloc, "[", 0, 2)

        ave <- c(mean(kidx), mean(kidy))
        midpt <- c((point[1]+ave[1])/2, (point[2]+ave[2])/2)
        sgn <- sign(ave[2] - point[2])

        segments(point[1], point[2]+sgn*gap, point[1], midpt[2],
                 lwd=lwd, col=col, ...)

        segments(min(kidx), midpt[2], max(kidx), midpt[2],
                 lwd=lwd, col=col, ...)

        arrows(kidx, rep(midpt[2], length(kidx)),
               kidx, kidy-sgn*(chrlength/2+gap),
               lwd=lwd, length=arrow_length, col=col, ...)
    }

    invisible(NULL)
}

#' Example AIL pedigree
#'
#' Example matrix describing the pedigree for advanced intercross lines
#'
#' @docType data
#'
#' @usage data(AILped)
#'
#' @format A data frame with five columns: individual id, mom, dad, sex (0
#' for females and 1 for males) and generation.
#'
#' @source Derived from the pedF8 dataset in the QTLRel package,
#' <https://cran.r-project.org/package=QTLRel>
#'
#' @keywords datasets
#'
#' @examples
#' data(AILped)
#' x <- sim_from_pedigree(AILped)
"AILped"

# sim_ail_pedigree
#
#' Simulate AIL pedigree
#'
#' Simulate a pedigree for advanced intercross lines (a table of
#' individual, mom, dad, sex)
#'
#' @param ngen Number of generations of outbreeding
#' @param npairs Number of breeding pairs at each generation
#' @param nkids_per Number of offspring per pair for the last
#' generation
#' @param design How to choose crosses: either random but avoiding
#' siblings, or completely at random
#'
#' @return A data frame with five columns: individual ID, mother ID,
#' father ID, sex, and generation.  Founders have `0` for mother
#' and father ID. Sex is coded 0 for female and 1 for male.
#'
#' @details Advanced intercross lines (AIL) are generated from a pair of inbred lines.
#' We cross them and then cross the F1 to generate `npair` breeding pairs.
#' The subsequent `ngen` outbreeding generations then proceed by
#' crossing a male and female from the preceding generation (mated
#' completely at random, with `design="random"`, or avoiding
#' siblings, with `design="nosib"`). Each breeding pair gives a
#' single female and a single male to the next generation, except at
#' the last generation `nkids_per` offspring are mated, in equal
#' numbers male and female. (If `nkids_per` is an odd number, the
#' number of males and females in each sibship will differ by one,
#' alternating between sibships, with one additional female and then
#' one additional male.
#'
#' @export
#' @keywords datagen
#' @seealso [sim_from_pedigree()],
#' [sim_ril_pedigree()], [sim_do_pedigree()],
#' [sim_4way_pedigree()]
#'
#' @examples
#' tab <- sim_ail_pedigree(12, 30)
sim_ail_pedigree <-
    function(ngen=12, npairs=30, nkids_per=5, design=c("nosib", "random"))
{
    design <- match.arg(design)

    # initial generations
    id <- c(1,2,3,4, (1:(npairs*2)+4))
    mom <- c(0,0,1,1,rep(3, npairs*2))
    dad <- c(0,0,2,2,rep(4, npairs*2))
    sex <- c(0,1,0,1,rep(c(0,1), npairs))
    gen <- c(0,0,1,1,rep(2, npairs*2))

    if(ngen <= 2)
        stop("No. generations should be > 2")

    cur_nind <- length(id)
    moms <- seq(5, length(id), by=2)
    dads <- seq(6, length(id), by=2)
    for(i in 3:ngen) {
        if(i > 3) {
            dads <- sample(dads)
            while(design=="nosib" && any(dads - moms == 1)) { # sample until no sibs
                dads <- sample(dads)
            }
        }

        if(i < ngen) {
            kids <- 1:(npairs*2)+max(id)

            id <- c(id, kids)
            mom <- c(mom, rep(moms, each=2))
            dad <- c(dad, rep(dads, each=2))
            sex <- c(sex, rep(c(0,1), npairs))
            gen <- c(gen, rep(i, npairs*2))

            moms <- kids[seq(1, length(kids), by=2)]
            dads <- kids[seq(2, length(kids), by=2)]
        }
        else { # last generation: expand
            kids <- 1:(npairs*nkids_per)+max(id)

            id <- c(id, kids)
            mom <- c(mom, rep(moms, each=nkids_per))
            dad <- c(dad, rep(dads, each=nkids_per))
            sex <- c(sex, rep(c(0,1), ceiling(npairs*nkids_per/2))[1:(npairs*nkids_per)])
            gen <- c(gen, rep(i, npairs*nkids_per))
        }
    }

    data.frame(id=id, mom=mom, dad=dad, sex=sex, gen=gen)
}

## where_het.R

# where_het
#
#' Find heterozygous regions
#'
#' Find regions of heterozygosity in an individual
#'
#' @param ind An individual object, as output be
#' [create_parent()] or [cross()]
#'
#' @return A matrix with two columns; each row indicates the start and
#' end of a region where the individual is heterozygous
#'
#' @export
#' @seealso [sim_from_pedigree()],
#' [convert2geno()]
#' @examples
#' mom <- create_parent(100, 1:2)
#' dad <- create_parent(100, 1:2)
#' child <- cross(mom, dad)
#' where_het(child)
where_het <-
    function(ind)
{
    if(length(ind$mat$locations)==length(ind$pat$locations) &&
       all(ind$mat$locations == ind$pat$locations) &&
       all(ind$mat$alleles == ind$pat$alleles)) {
        het <- matrix(ncol=2, nrow=0)
        dimnames(het) <- list(NULL, c("left", "right"))
        return(het)
    }

    u <- c(0, sort(unique(c(ind$mat$locations,ind$pat$locations))))
    het <- NULL
    for(i in 2:length(u)) {
        toright <- which(ind$mat$locations >= u[i])
        mat <- ind$mat$alleles[toright[1]]

        toright <- which(ind$pat$locations >= u[i])
        pat <- ind$pat$alleles[toright[1]]

        if(mat!=pat) { # heterozygous
            if(is.null(het)) het <- cbind(u[i-1],u[i])
            else het <- rbind(het,c(u[i-1],u[i]))
        }
    }

    # clean up
    if(nrow(het) > 1) {
        keep <- rep(TRUE,nrow(het))
        for(j in 2:nrow(het)) {
            if(het[j,1] == het[j-1,2]) {
                het[j,1] <- het[j-1,1]
                keep[j-1] <- FALSE
            }
        }
        het <- het[keep,,drop=FALSE]
    }

    dimnames(het) <- list(1:nrow(het), c("left", "right"))
    het
}

# convert2geno:
#
#' Convert continuous allele information into marker genotypes
#'
#' Convert the continuous crossover location information produced by
#' sim_from_pedigree to marker genotypes
#'
#' @param xodat The sort of detailed genotype/crossover data generated by
#' [sim_from_pedigree()]
#' @param map vector of marker locations; can also be a list of such
#' vectors (one per chromosome), in which case xodat and founder_geno
#' must be lists with the same length.
#' @param founder_geno Optional matrix (size `n_founders` x
#' `length(map)`) of founder genotypes. If coded as 1/2 (or 1/3),
#' results are 1/2/3 genotypes. If coded as A/T/G/C/N, results are
#' A/T/G/C/N/H genotypes. If coded as letters A-H for the 8 founders,
#' results are two-letter genotypes AA-HH with 36 possible values.
#' @param shift_map If TRUE, shift genetic map to start at 0
#'
#' @return If `founder_geno` is provided or there are just two
#' founders, the result is a numeric matrix of genotypes, individuals
#' x markers, with genotypes 1/2/3 codes for 11/12/22 genotypes.
#'
#' If `founder_geno` is not provided and there are more than two
#' founders, the result is a 3-dimensional array, individuals x
#' markers x alleles, with the third dimensional corresponding to the
#' maternal and paternal allele.
#'
#' If the input `map` is a list (the components being
#' chromosomes), then `xodat` and `founder_geno` must be
#' lists of the same length, and the result will be a list of
#' matrices.
#'
#' @export
#' @keywords utilities
#' @seealso [get_geno()], [sim_from_pedigree()]
#'
#' @examples
#' # simulate AIL pedigree
#' tab <- sim_ail_pedigree(12, 30)
#' # simulate data from that pedigree
#' dat <- sim_from_pedigree(tab)
#' # marker map (could also use sim.map in R/qtl)
#' map <- seq(0, 100, by=5)
#' names(map) <- paste0("marker", seq(along=map))
#' # convert data to marker genotypes
#' geno <- convert2geno(dat, map)
#'
#'
#' # AIL with multiple chromosomes
#' dat <- sim_from_pedigree(tab, c("1"=100, "2"=75, "X"=100), xchr="X")
#' # marker map
#' multmap <- list("1"=seq(0, 100, by=5),
#'                 "2"=seq(0, 75, by=5),
#'                 "X"=seq(0, 100, by=5))
#' for(i in 1:3)
#'   names(multmap[[i]]) <- paste0("marker", i, "_", 1:length(map[[i]]))
#' geno <- convert2geno(dat, multmap)
#'
#' # simulate DO pedigree
#' tab <- sim_do_pedigree(8)
#' # simulate data from that pedigree
#' dat <- sim_from_pedigree(tab)
#' # simulate founder snp alleles
#' fg <- matrix(sample(1:2, 8*length(map), repl=TRUE), nrow=8)
#' # for DO, need female & male founders (to deal with X chr)
#' fg <- rbind(fg, fg)
#' # convert dat to SNP genotypes
#' geno <- convert2geno(dat, map, fg)
#' # if fg not provided, result is a 3d array
#' genoarray <- convert2geno(dat, map)
convert2geno <-
    function(xodat, map, founder_geno=NULL, shift_map=FALSE)
{
    if(is.list(map)) { # map is a list of multiple chromosomes
        if(length(xodat) != length(map))
            stop("length(xodat) != length(map)")
        if(!is.null(founder_geno)) {
            if(length(founder_geno) != length(map))
                stop("length(founder_geno) != length(map)")
        }
        result <- vector("list", length(map))
        names(result) <- names(map)
        for(i in seq(along=map))
            result[[i]] <- convert2geno(xodat[[i]], map[[i]],
                                        founder_geno[[i]], shift_map)
        return(result)
    }

    if(shift_map) map <- map - map[1]
    if(max(map) > max(xodat[[1]]$mat$locations))
        stop("maximum simulated position is less than the length of the map.")
    if(any(map < 0))
        stop("Some markers at < 0 cM")

    # maximum founder genotype
    max_geno <- max(sapply(xodat, function(a) max(a$mat$alleles, a$pat$alleles)))

    if(is.null(founder_geno))
        founder_geno <- matrix(nrow=0, ncol=0)
    else {
        if(nrow(founder_geno) < max_geno)
            stop("founder_geno should have at least ", max_geno,
                 " rows but has ", nrow(founder_geno))
        if(ncol(founder_geno) != length(map))
            stop("founder_geno should have ", length(map),
                 " columns but has ", ncol(founder_geno))
        if(any(is.na(founder_geno) | founder_geno==0)) # 0 as missing
            stop("founder genotypes cannot be missing")
    }

    if(length(founder_geno) > 0 || max_geno <= 2) {
        output <- t(.convert2geno(xodat, map, founder_geno))

        if(any(founder_geno != 1 & founder_geno != 2)) {
            if(all(founder_geno == 1 | founder_geno==3)) {
                founder_geno[founder_geno==3] <- 2
                output <- t(.convert2geno(xodat, map, founder_geno))
            }
            else if(all(c("A","T","G","C") %in% unique(c(founder_geno)))){
                output <- t(convert2geno_char(xodat, map, founder_geno))
            } else {
                output <- t(convert2geno_char_paste(xodat, map, founder_geno))
            }
        } else {
            output <- t(.convert2geno(xodat, map, founder_geno))
        }

        dimnames(output) <- list(names(xodat), names(map))
    }
    else {
        output <- convert2genoarray(xodat, map)
        # output was markers x ind x alleles; switch first two dimensions
        output <- aperm(output, c(2, 1, 3))

        dimnames(output) <- list(names(xodat), names(map), c("mat", "pat"))
    }

    output
}

#' Mouse chromosome lengths
#'
#' Mouse chromosome lengths in cM from the Cox et al. map
#'
#' @docType data
#'
#' @usage data(mouseL_cox)
#'
#' @format A numeric vector with lengths in cM for the 20 mouse chromosomes.
#'
#' @source Taken from Table 1 of Cox et al. (2009) A new standard
#'     genetic map for the laboratory mouse. Genetics 182:1335-1344.
#'     \doi{10.1534/genetics.109.105486}
#'
#' @keywords datasets
#'
#' @seealso mouseL_mgi
#'
#' @examples
#' data(mouseL_cox)
"mouseL_cox"

#' Simulate AIL pedigree with fixed n
#'
#' Simulate a pedigree for advanced intercross lines (a table of
#' individual, mom, dad, sex) so that the last generation reaches a
#' desired sample size n
#'
#' @param ngen Number of generations of outbreeding
#' @param npairs Number of breeding pairs at each generation. If
#' missing, we use 30 when `method="last2"` and 300 when
#' `method="sub2"`.
#' @param nkids_per Number of offspring per pair for the last
#' generation
#' @param design How to choose crosses: either random but avoiding
#' siblings, or completely at random
#' @param method Method used to generate pedigree: either expand at the last
#' two generations or generate a pedigree with a large number of pairs and
#' select a subset to have the desired sample size.
#' @param nsample_ngen Number of individuals desired at the last
#' generation
#'
#' @details The default value for `npairs` depends on the choice of `method`.
#' For `method="last2"`, we use a default of `npairs=30`; for
#' `method="sub2"`, we use a default of `npairs=300`.
#'
#' @return A data frame with five columns: individual ID, mother ID,
#' father ID, sex, and generation.  Founders have `0` for mother
#' and father ID. Sex is coded 0 for female and 1 for male.
#'
#'
#' @export
#' @keywords datagen
#' @seealso [sim_from_pedigree()],
#' [sim_ril_pedigree()], [sim_ail_pedigree()],
#' [sim_do_pedigree()], [sim_4way_pedigree()],
#' [sim_do_pedigree_fix_n()]
#'
#' @examples
#' tab <- sim_ail_pedigree_fix_n(12)
sim_ail_pedigree_fix_n <- function(ngen=12, nkids_per=5,
                                  nsample_ngen=150, npairs=NULL,
                                  method=c("last2", "sub2"),
                                  design=c("nosib", "random")){
  method <- match.arg(method)
  design <- match.arg(design)

  if(method =="last2"){
    if(is.null(npairs))
      npairs <- 30
    ped <- sim_ail_pedigree_last2(ngen = ngen, npairs = npairs, nkids_per = nkids_per,
                                  design = design, nsample_ngen = nsample_ngen)
    id <- which(ped[, "gen"] == ngen)
  } else if(method =="sub2"){
    if(is.null(npairs))
      npairs <- 300
    npairs.selc <- nsample_ngen / nkids_per
    ped <- sim_ail_pedigree(ngen = ngen, npairs = npairs,
                            nkids_per = nkids_per, design = design)
    selc.dad <- sample(which(ped[, "gen"] == ngen-1 & ped[, "sex"] == 1),
                       size=npairs.selc)
    id <- which(ped[, "gen"] == ngen & ped[,"dad"] %in% selc.dad)
  }
  attr(ped, "last.gen.id") <- id
  ped
}

sim_ail_pedigree_last2 <- function(ngen=12, npairs=30, nkids_per=5, design=c("nosib", "random"),
                                   nsample_ngen=150)
{
  design <- match.arg(design)
  npairs_la2 <- ceiling(nsample_ngen/nkids_per)
  nkids_la2 <- ceiling(npairs_la2*2/npairs)

  ## second-to-last generation
  ped <- sim_ail_pedigree(ngen=ngen-1, npairs=npairs, nkids_per=nkids_la2,
                          design=design)
  id <- ped[,"id"]
  mom <- ped[,"mom"]
  dad <- ped[,"dad"]
  sex <- ped[,"sex"]
  gen <- ped[,"gen"]

  ## last generation
  n.last <- npairs_la2*nkids_per
  kids <- 1:(n.last)+max(id)

  wh <- which(ped[,"gen"] == ngen-1 & ped[, "sex"] == 0)
  moms <- ped[wh, "id"]
  wh <- which(ped[,"gen"] == ngen-1 & ped[, "sex"] == 1)
  dads <- ped[wh, "id"]
  rownames(ped) <- ped[, "id"]

  while(design=="nosib") { # sample until no sibs
    dads <- sample(dads)
    if(all(ped[dads, "dad"] != ped[moms, "dad"]))
        break
  }

  wh <- sort(sample(n.last, size=nsample_ngen))
  id <- c(id, kids[wh])
  mom <- c(mom, rep(moms, each=nkids_per)[wh])
  dad <- c(dad, rep(dads, each=nkids_per)[wh])
  sex <- c(sex, rep_len(c(0,1), length.out=n.last)[wh])
  gen <- c(gen, rep(ngen, n.last)[wh])

  data.frame(id=id, mom=mom, dad=dad, sex=sex, gen=gen)
}

## sim_ril.R

# these functions aren't very good
sim_ri2 <-
    function(L, n.gen=20, m=10, p=0, selfing=FALSE,
             obligate_chiasma=FALSE)
{
    if(obligate_chiasma) Lstar <- calc_Lstar(L, m, p)
    else Lstar <- L

    f1 <- create_parent(L,c(1,2))
    par1 <- cross(f1,f1,m,p)

    if(selfing) par2 <- par1
    else par2 <- cross(f1,f1,m,p, obligate_chiasma, Lstar)

    for(i in 1:n.gen) {
        c1 <- cross(par1,par2,m,p, obligate_chiasma, Lstar)
        if(selfing)
            par1 <- par2 <- c1
        else {
            c2 <- cross(par1,par2,m,p, obligate_chiasma, Lstar)
            par1 <- c1
            par2 <- c2
        }
    }
    par1
}

sim_ri8 <-
    function(L, n.gen=20, m=10, p=0, selfing=FALSE,
             obligate_chiasma=FALSE)
{
    if(obligate_chiasma) Lstar <- calc_Lstar(L, m, p)
    else Lstar <- L

    f1a <- create_parent(L,c(1,2))
    f1b <- create_parent(L,c(3,4))
    f1c <- create_parent(L,c(5,6))
    f1d <- create_parent(L,c(7,8))
    par1 <- cross(f1a,f1b,m,p, obligate_chiasma, Lstar)

    if(selfing) par2 <- par1
    else par2 <- cross(f1c,f1d,m,p, obligate_chiasma, Lstar)

    if(length(n.gen)==1) {
        for(i in 1:(n.gen+1)) {
            c1 <- cross(par1,par2,m,p, obligate_chiasma, Lstar)
            if(selfing)
                par1 <- par2 <- c1
            else {
                c2 <- cross(par1,par2,m,p, obligate_chiasma, Lstar)
                par1 <- c1
                par2 <- c2
            }
        }
        return(par1)
    }
    else {
        result <- vector("list",length(n.gen))
        names(result) <- n.gen
        n.gen <- c(-1,n.gen)
        for(j in 2:length(n.gen)) {
            for(i in (n.gen[j-1]+2):(n.gen[j]+1)) {
                c1 <- cross(par1,par2,m,p, obligate_chiasma, Lstar)
                if(selfing)
                    par1 <- par2 <- c1
                else {
                    c2 <- cross(par1,par2,m,p, obligate_chiasma, Lstar)
                    par1 <- c1
                    par2 <- c2
                }
            }
            result[[j-1]] <- par1
        }
        return(result)
    }

}

#' Simulate a pedigree for Diversity Outbreds for a target sample size
#'
#' Simulate a pedigree for Diversity Outbred (DO) mice (a table of
#' individual, mom, dad, sex) so that the last generation reaches a
#' desired sample size.
#'
#' @param ngen Number of generations of outbreeding
#' @param npairs Number of breeding pairs at each generation. If
#' missing, we use 30 when `method="last2"` and 300 when
#' `method="sub2"`.
#' @param nkids_per Number of offspring per pair for the last
#' generation
#' @param design How to choose crosses: either random but avoiding
#' siblings, or completely at random
#' @param method Method used to generate the pedigree: either expand
#' at the last two generations or generate a pedigree with a large
#' number of pairs and then select a subset to have the desired sample
#' size. With `method="fixcc"`, we use the pre-CC generations as performed at the
#' Jackson Lab.
#' @param selc.method Method used to select the individuals from last
#' generation.
#' @param nsample_ngen Number of individuals desired at the last
#' generation
#' @param nccgen The number of generations for each CC line, only used
#' when `method` is not `"fixcc"`.
#'
#' @details The default number of breeding pairs depends on the chosen
#' `method`. With `method="last2"`, the default is `npairs=30`;
#' with `method="sub2"`, the default is `npairs=300`;
#' with `method="fixcc"`, `npairs` is ignored and is fixed at 144.
#'
#' @return A data frame with six columns: individual ID, mother ID, father
#' ID, sex, generation, and TRUE/FALSE indicator for whether DO or pre-DO.
#' Founders have `0` for mother and father ID. Sex is coded 0 for
#' female and 1 for male.
#'
#' @export
#' @seealso [sim_from_pedigree()],
#' [sim_ril_pedigree()], [sim_ail_pedigree()],
#' [sim_do_pedigree()], [sim_4way_pedigree()],
#' [sim_ail_pedigree_fix_n()]
#'
#' @examples
#' tab <- sim_do_pedigree_fix_n(8)
sim_do_pedigree_fix_n <- function(ngen=12, nkids_per=5, nccgen=15,
                                  nsample_ngen=150, npairs=NULL,
                                  method=c("last2", "sub2", "fixcc"),
                                  design=c("nosib", "random"),
                                  selc.method=c("byfamily","byindiv")){
  method <- match.arg(method)
  design <- match.arg(design)
  selc.method <- match.arg(selc.method)

  if(method =="last2"){
    if(is.null(npairs))
      npairs <- 30
    ped <- sim_do_pedigree_last2(ngen = ngen, npairs = npairs, nkids_per = nkids_per,
                                 nccgen = nccgen, design = design,
                                 nsample_ngen = nsample_ngen)
  } else if(method =="sub2"){
    if(is.null(npairs))
      npairs <- 300
    ped <- sim_do_pedigree(ngen = ngen, npairs = npairs,
                           ccgen = rep(nccgen, npairs),
                           nkids_per = nkids_per, design = design)
  } else if(method=="fixcc"){
    ## use fixed ccgen value
    alpha <- c(21, 64, 24, 10, 5, 9, 5, 3, 3)
    names(alpha) <- 4:12
    npairs <- sum(alpha)
    ccgen <- rep(as.numeric(names(alpha)), alpha)
    ped <- sim_do_pedigree(ngen = ngen, npairs = npairs, ccgen=ccgen,
                           nkids_per = nkids_per, design = design)
  }

  ## selecting samples...
  if(method == "last2"){
    id <- which(ped[, "gen"] == ngen & ped[,"do"]==1)
  } else {
    if(selc.method == "byfamily"){
      npairs.selc <- nsample_ngen / nkids_per
      selc.dad <- sample(which(ped[, "gen"] == ngen-1 &
                               ped[, "sex"] == 1 & ped[,"do"]==1),
                         size=npairs.selc)
      id <- which(ped[,"dad"] %in% selc.dad &
                  ped[, "gen"] == ngen & ped[,"do"]==1)
    }else{
      id <- ped[ped[, "gen"] == ngen & ped[, "do"] == 1, "id"]
      id <- sample(id, nsample_ngen)
    }
  }

  attr(ped, "last.gen.id") <- id
  ped
}


sim_do_pedigree_last2 <- function(ngen=12, npairs=30, nkids_per=5, nccgen=15,
                                  design=c("nosib", "random"),
                                  nsample_ngen=150)
{
  design <- match.arg(design)
  npairs_la2 <- ceiling(nsample_ngen/nkids_per)
  nkids_la2 <- ceiling(npairs_la2*2/npairs)

  ## second-to-last generation
  ped <- sim_do_pedigree(ngen = ngen-1, npairs = npairs,
                         ccgen = rep(nccgen, npairs),
                         nkids_per=nkids_la2, design=design)
  id <- ped[,"id"]
  mom <- ped[,"mom"]
  dad <- ped[,"dad"]
  sex <- ped[,"sex"]
  gen <- ped[,"gen"]
  do <- ped[,"do"]

  ## last generation
  n.last <- npairs_la2*nkids_per
  kids <- 1:(n.last)+max(id)

  wh <- which(ped[,"gen"] == ngen-1 & ped[, "sex"] == 0 & ped[, "do"] == 1)
  moms <- ped[wh, "id"]
  wh <- which(ped[,"gen"] == ngen-1 & ped[, "sex"] == 1 & ped[, "do"] == 1)
  dads <- ped[wh, "id"]
  rownames(ped) <- ped[, "id"]

  while(design=="nosib") { # sample until no sibs
    dads <- sample(dads)
    if(all(ped[dads, "dad"] != ped[moms, "dad"]))
        break
  }

  wh <- sort(sample(n.last, size=nsample_ngen))
  id <- c(id, kids[wh])
  mom <- c(mom, rep(moms, each=nkids_per)[wh])
  dad <- c(dad, rep(dads, each=nkids_per)[wh])
  sex <- c(sex, rep_len(c(0,1), length.out=n.last)[wh])
  gen <- c(gen, rep(ngen, n.last)[wh])
  do <- c(do, rep(1, length(wh)))

  data.frame(id=id, mom=mom, dad=dad, sex=sex, gen=gen, do=do)
}

#' Convert continuous allele information into marker genotypes for
#' multiple chromosomes
#'
#' Wrap up of convert2geno to adequate multiple chromosomes.
#'
#' @param xodat The sort of detailed genotype/crossover data generated
#' by [sim_from_pedigree_allchr()]
#' @param map marker locations, a list with elements for each
#' chromosome
#' @param id ids for which individuals genotypes is desired
#' @param founder_geno Optional list of matrices (one per chromosome)
#' of size `n_founders` x `n_markers`, with the founder genotypes.
#' If coded as 1/2 (or 1/3), results are 1/2/3
#' genotypes. If coded as A/T/G/C/N, results are A/T/G/C/N/H
#' genotypes. If coded as letters A-H (in the case of 8 founders), results are
#' two-letter genotypes AA-HH with 36 possible values.
#' @param shift_map If TRUE, shift genetic map to start at 0
#' @param return.matrix If FALSE, the result is a list of length
#' `n_chrs`, otherwise it is converted into a matrix if size
#' `length(id)` x `n_markers`.
#'
#' @return If `founder_geno` is provided or there are just two
#' founders, the result is a numeric matrix of genotypes, individuals
#' x markers, with genotypes 1/2/3 codes for 11/12/22 genotypes. If
#' there are more than two founders and `founder_geno` are
#' letters, the result is a character matrix, too.
#'
#' If `founder_geno` is not provided and there are more than two
#' founders, the result is a 3-dimensional array, individuals x
#' markers x alleles, with the third dimensional corresponding to the
#' maternal and paternal allele.
#'
#' @export
#' @keywords utilities
#' @seealso [convert2geno()]
#'
#' @examples
#' library(qtl)
#' # marker map
#' map <- sim.map(len=rep(100, 19), n.mar=10, include.x=FALSE)
#' # simulate AIL pedigree
#' tab <- sim_ail_pedigree(12, 30)
#' # simulate data from that pedigree
#' dat <- sim_from_pedigree_allchr(tab, map)
#' names(map) <- paste0("marker", seq(along=map))
#' # convert data to marker genotypes
#' id <- which(tab[, "gen"]==12)
#' geno <- convert2geno_allchr(dat, map, id)
#'
convert2geno_allchr <- function(xodat, map, id=NULL, founder_geno=NULL,
                                  return.matrix=TRUE, shift_map=FALSE){

  stopifnot(length(map) == length(xodat))
  if(is.null(id)) id <- 1:length(xodat[[1]])

  if(is.null(founder_geno)){
    geno <- NULL
    for(chr in 1:length(map)){
      geno[[chr]] <- convert2geno(xodat=xodat[[chr]][id], map=map[[chr]])
    }
    names(geno) <- names(map)
  }else{

    ## create a list of founder geno, one element for each chr.
    if(is.list(founder_geno)){
      ## check that founder_geno is a list of same length as map
      stopifnot(length(founder_geno)==length(map))
    }else{  ## founder_geno is a matrix.
      nm.chr <- sapply(map, length)
      nm.map <- sum(nm.chr)
      nm <- nrow(founder_geno)
      if(nm.map!=nm) stop("founder_geno should have ", nm.map, " columns but has ", nm)

      founder_geno <- cbind(founder_geno, founder_geno)
      founder_geno.list <- list()
      for(chr in 1:length(map)){
        if(chr==1)
            founder_geno.list[[chr]] <- t(founder_geno[1:nm.chr[chr], ])
        else
            founder_geno.list[[chr]] <- t(founder_geno[sum(nm.chr[1:(chr-1)]) + (1:nm.chr[chr]), ])
      }
      founder_geno <- founder_geno.list
    }
    geno <- NULL
    for(chr in 1:length(map)){
      geno[[chr]] <- convert2geno(xodat=xodat[[chr]][id], map=map[[chr]],
                                  founder_geno=founder_geno[[chr]])
    }
    names(geno) <- names(map)
  }

  if(!return.matrix){ ## return the list directly
    return(geno)
  }else{ ## cbind all elememts to make a matrix.
    if(is.matrix(geno[[1]])){
      geno.mat <- NULL
      for(i in 1:length(geno)){
        geno.mat <- cbind(geno.mat, geno[[i]])
      }
    }else{ ## this could be a arrary when founder_geno is missing...
      ## individuals x markers x alleles,
      geno.mat1 <- NULL
      geno.mat2 <- NULL
      for(i in 1:length(geno)){
        geno.mat1 <- cbind(geno.mat1, geno[[i]][,,1])
        geno.mat2 <- cbind(geno.mat2, geno[[i]][,,2])
      }
      geno.mat <- array(NA, dim=c(dim(geno.mat1), 2))
      geno.mat[,,1] <- geno.mat1
      geno.mat[,,2] <- geno.mat2
    }
    return(geno.mat)
  }
}

## sim_meiosis.R

# create_parent
#
#' Create a parent object
#'
#' Create a parent object
#'
#' @param L chromosome length in cM
#' @param allele vector of integers for alleles, of length 1 or 2
#'
#' @return A list with two components, for the individual's two
#' chromosomes.  Each is a list with alleles in chromosome intervals
#' (as integers) and locations of the right endpoints of those
#' intervals.
#'
#' @keywords datagen
#' @export
#' @seealso [cross()], [sim_meiosis()]
#'
#' @examples
#' create_parent(100, 1)
#' create_parent(100, 1:2)
create_parent <-
    function(L, allele=1)
{
    if(length(allele) == 1) allele <- rep(allele,2)
    if(length(allele) != 2)
        stop("allele should be of length 1 or 2")
    if(!is.integer(allele)) {
        if(!is.numeric(allele))
            stop("allele should be a vector with 1 or 2 integers")
        storage.mode(allele) <- "integer"
    }

    list(mat=list(alleles=allele[1], locations=L),
         pat=list(alleles=allele[2], locations=L))
}


# check that data for an individual conforms to expected format
check_individual <-
    function(ind, tol=1e-12)
{
    # list with two components, named "mat" and "pat"
    if(!is.list(ind) || length(ind)!=2 ||
       !all(names(ind) == c("mat", "pat")))
        stop('ind should be list with "mat" and "pat"')

    # check each chromosome
    for(i in 1:2) {
        # list with "alleles" and "locations" components
        if(!is.list(ind[[i]]) || length(ind[[i]])!=2 ||
           !all(names(ind[[i]]) == c("alleles", "locations")))
            stop('chromosome should be list with "alleles" and "locations"')

        alleles <- ind[[i]]$alleles
        locations <- ind[[i]]$locations

        # locations numeric
        if(!is.numeric(locations))
            stop("locations should be numeric")

        # alleles integer
        if(!is.integer(alleles))
            stop("alleles should be integers")

        # locations non-decreasing
        if( length(locations) > 1 && min(diff(locations)) < 0 )
            stop("locations should be non-decreasing")
    }

    # start and end positions are the same
    starts <- vapply(ind, function(a) a$location[1], 0.0)
    ends <- vapply(ind, function(a) a$location[length(a$location)], 0.0)
    stopifnot(abs(diff(starts)) < tol, abs(diff(ends)) < tol)

    TRUE
}

#' Simulate crossover locations using the Stahl model
#'
#'
#' Simulate crossover locations on a single meiotic product using the
#' Stahl model.
#'
#' @details Chiasma locations are a superposition of two
#' processes: a proportion p exhibiting no interference, and a
#' proportion (1-p) following the chi-square model with interference
#' parameter m.  Crossover locations are derived by thinning the
#' chiasma locations with probability 1/2.
#'
#' @param L length of chr in cM
#' @param m Interference parameter (`m=0` is no interference)
#' @param p Proportion of chiasmata from no-interference mechanism
#' (`p=0` gives pure chi-square model)
#' @param obligate_chiasma If TRUE, require an obligate chiasma on the
#' 4-strand bundle at meiosis.
#' @param Lstar Adjusted chromosome length, if
#' `obligate_chiasma=TRUE`. Calculated if not provided.
#'
#' @return Numeric vector of crossover locations, in cM
#'
#' @keywords datagen
#'
#' @details Simulations are under the Stahl model with the
#' interference parameter being an integer. This is an extension of
#' the chi-square model, but with chiasmata being the superposition of
#' two processes, one following the chi-square model and the other
#' exhibiting no interference.
#'
#' @examples
#' x <- sim_crossovers(200, 10, 0)
#' x <- sim_crossovers(200, 10, 0.04)
#' x <- sim_crossovers(100, 0, 0, obligate_chiasma=TRUE)
#'
#' @references
#' Copenhaver, G. P., Housworth, E. A. and Stahl, F. W. (2002) Crossover
#' interference in arabidopsis.  \emph{Genetics} \bold{160}, 1631--1639.
#'
#' Foss, E., Lande, R., Stahl, F. W. and Steinberg, C. M. (1993) Chiasma
#' interference as a function of genetic distance. \emph{Genetics}
#' \bold{133}, 681--691.
#'
#' Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis
#' of crossover interference using the chi-square model.  \emph{Genetics}
#' \bold{139}, 1045--1056.
#'
#' @export
#'
sim_crossovers <-
    function(L, m=10, p=0.0, obligate_chiasma=FALSE, Lstar=NULL)
{
    if(is.null(Lstar)) {
        if(obligate_chiasma)
            Lstar <- calc_Lstar(L, m, p)
        else Lstar <- L
    }

    .sim_crossovers(L, m, p, obligate_chiasma, Lstar)
}


#' Simulate meiosis
#'
#' Output a random meiotic product from an input individual.
#'
#' @param parent An individual object, as output by
#' [create_parent()] or [cross()]
#' @param m interference parameter for chi-square model
#' @param p Proportion of chiasmata coming from no-interference process.
#' @param obligate_chiasma If TRUE, require an obligate chiasma on the
#' 4-strand bundle at meiosis.
#' @param Lstar Adjusted chromosome length, if
#' `obligate_chiasma=TRUE`. Calculated if not provided.
#'
#' @return A list with alleles in chromosome intervals (as integers)
#' and locations of the right endpoints of those intervals.
#'
#' @details Simulations are under the Stahl model with the
#' interference parameter being an integer. This is an extension of
#' the chi-square model, but with chiasmata being the superposition of
#' two processes, one following the chi-square model and the other
#' exhibiting no interference.
#'
#' @references
#' Copenhaver, G. P., Housworth, E. A. and Stahl, F. W. (2002) Crossover
#' interference in arabidopsis.  \emph{Genetics} \bold{160}, 1631--1639.
#'
#' Foss, E., Lande, R., Stahl, F. W. and Steinberg, C. M. (1993) Chiasma
#' interference as a function of genetic distance. \emph{Genetics}
#' \bold{133}, 681--691.
#'
#' Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis
#' of crossover interference using the chi-square model.  \emph{Genetics}
#' \bold{139}, 1045--1056.
#'
#' @keywords datagen
#' @export
#' @seealso [create_parent()], [cross()],
#' [sim_crossovers()], [calc_Lstar()]
#'
#' @examples
#' ind <- create_parent(100, 1:2)
#' prod <- sim_meiosis(ind)
sim_meiosis <-
    function(parent, m=10, p=0.0, obligate_chiasma=FALSE, Lstar=NULL)
{
    if(is.null(Lstar)) {
        L <- max(parent$mat$locations)
        if(obligate_chiasma)
            Lstar <- calc_Lstar(L, m, p)
        else Lstar <- L
    }

    .sim_meiosis(parent, m, p, obligate_chiasma, Lstar)
}


# cross
#
#' Cross two individuals
#'
#' Simulate the cross of two individuals to create a
#' single progeny
#'
#' @param mom An individual object, as produced by
#' [create_parent()] or this function.
#' @param dad An individual object, as produced by
#' [create_parent()] or this function.
#' @param m interference parameter for chi-square model
#' @param p proportion of crossovers coming from no-interference process
#' @param xchr If TRUE, simulate X chromosome
#' @param male If TRUE, simulate a male (matters only if
#' `xchr=TRUE`)
#' @param obligate_chiasma If TRUE, require an obligate chiasma on the
#' 4-strand bundle at meiosis.
#' @param Lstar Adjusted chromosome length, if
#' `obligate_chiasma=TRUE`. Calculated if not provided.
#'
#' @details Simulations are under the Stahl model with the
#' interference parameter being an integer. This is an extension of
#' the chi-square model, but with chiasmata being the superposition of
#' two processes, one following the chi-square model and the other
#' exhibiting no interference.
#'
#' @return A list with two components, for the individual's two
#' chromosomes.  Each is a list with alleles in chromosome intervals
#' (as integers) and locations of the right endpoints of those
#' intervals.
#'
#' @keywords datagen
#' @export
#' @seealso [create_parent()], [sim_meiosis()],
#' [sim_crossovers()], [calc_Lstar()]
#'
#' @examples
#' mom <- create_parent(100, 1:2)
#' dad <- create_parent(100, 1:2)
#' child <- cross(mom, dad)
cross <-
    function(mom, dad, m=10, p=0, xchr=FALSE, male=FALSE,
             obligate_chiasma=FALSE, Lstar=NULL)
{
    if(is.null(Lstar)) {
        L <- max(mom$mat$locations)
        if(obligate_chiasma)
            Lstar <- calc_Lstar(L, m, p)
        else Lstar <- L
    }

    if(!xchr) {
        return(list(mat=sim_meiosis(mom, m, p, obligate_chiasma, Lstar),
                    pat=sim_meiosis(dad, m, p, obligate_chiasma, Lstar)))
    }
    else {
        if(male)
            return(list(mat=sim_meiosis(mom, m, p, obligate_chiasma, Lstar),
                        pat=dad$pat))
        else
            return(list(mat=sim_meiosis(mom, m, p, obligate_chiasma, Lstar),
                        pat=dad$mat))
    }
}

#' Calculate adjusted chromosome length for obligate chiasma
#'
#' Calculate the reduced chromosome length that will give the target
#' expected number of chiasmata when conditioning on there being at
#' least one chiasma on the four-strand bundle.
#'
#' @param L Length of chromosome (in cM); must be > 50
#' @param m Interference parameter for chi-square model
#' @param p Proportion of chiasmata coming from no-interference
#' process
#'
#' @return Adjusted length of chromosome
#'
#' @keywords utilities
#' @importFrom stats uniroot dpois
#' @export
#' @seealso [cross()], [sim_meiosis()],
#' [sim_crossovers()]
#'
#' @examples
#' calc_Lstar(100, 0, 0)
#' calc_Lstar(60, 10, 0.1)
calc_Lstar <-
    function(L, m=0, p=0)
{
    if(L <= 50) stop("Must have L > 50")
    if(m < 0) stop("Must have m==0")
    if(!is.integer(m) && m %% 1 > 1e-8) {
        warning("m must be an non-negative integer; rounding")
        m <- round(m)
    }
    if(p < 0 || p > 1)
        stop("p must be in [0, 1]")
    if(p==1) { # if p == 1, might as well take m=0, p=0
        m <- 0
        p <- 0
    }

    func_to_zero <- function(Lstar, L, m=0, p=0) {
        if(m==0)
            denom <- 1 - exp(-Lstar/50)
        else {
            lambda1 <- Lstar/50 * (m+1) * (1-p)
            lambda2 <- Lstar/50 * p
            denom <- 1 - sum(dpois(0:m, lambda1) * (m+1 - (0:m))/(m+1)) * exp(-lambda2)
        }

        2*L - 2*Lstar / denom
    }

    uniroot(func_to_zero, c(1e-8, L), L=L, m=m, p=p,
            tol=sqrt(.Machine$double.eps))$root
}

context("meiosis")

test_that("create_parent works", {

    expected_p1 <- list(mat=list(alleles=1L, locations=100.0),
                        pat=list(alleles=1L, locations=100.0))

    expect_equal(create_parent(100, 1), expected_p1)

    expected_p2 <- list(mat=list(alleles=2L, locations=100.0),
                        pat=list(alleles=2L, locations=100.0))

    expect_equal(create_parent(100, 2), expected_p2)

    expected_f1 <- list(mat=list(alleles=1L, locations=100.0),
                        pat=list(alleles=2L, locations=100.0))

    expect_equal(create_parent(100, 1:2), expected_f1)

    # numeric alleles converted to integers
    expect_equal(create_parent(100, c(1.9, 2.9)), expected_f1)

    # non-numeric alleles not allowed
    expect_error(create_parent(100, "a"))

    # can't have more than 2 alleles
    expect_error(create_parent(100, 1:3))

    # must have at least 1 allele
    expect_error(create_parent(100, NULL))
    expect_error(create_parent(100, numeric(0)))

})


test_that("check_individual works", {

    expected_p1 <- list(mat=list(alleles=1L, locations=100.0),
                        pat=list(alleles=1L, locations=100.0))

    expected_p2 <- list(mat=list(alleles=2L, locations=100.0),
                        pat=list(alleles=2L, locations=100.0))

    expected_f1 <- list(mat=list(alleles=1L, locations=100.0),
                        pat=list(alleles=2L, locations=100.0))

    # should be clean
    expect_true(check_individual(expected_p1))
    expect_true(check_individual(expected_p2))
    expect_true(check_individual(expected_f1))

    # no data
    expect_error(check_individual(NULL))
    expect_error(check_individual(list(mat=NULL, pat=NULL)))

    # alleles not integers
    z <- list(mat=list(alleles=1, locations=100),
              pat=list(alleles=2, locations=100))
    expect_error(check_individual(z))
    z <- list(mat=list(alleles="a", locations=100),
              pat=list(alleles="b", locations=100))
    expect_error(check_individual(z))

    # wrong orders
    z <- list(mat=list(locations=100, alleles=1L),
              pat=list(alleles=1L, locations=100))
    expect_error(check_individual(z))
    z <- list(mat=list(alleles=1L, locations=100),
              pat=list(locations=100, alleles=1L))
    expect_error(check_individual(z))
    z <- list(pat=list(alleles=2L, locations=100),
              mat=list(alleles=1L, locations=100))
    expect_error(check_individual(z))

    # locations not in order
    z <- list(mat=list(alleles=c(2L, 1L), locations=c(5, 100)),
              pat=list(alleles=c(1L, 2L), locations=c(100, 85)))
    expect_error(check_individual(z))
    z <- list(mat=list(alleles=c(2L, 1L), locations=c(500, 100)),
              pat=list(alleles=c(1L, 2L), locations=c(100, 105)))
    expect_error(check_individual(z))
})

test_that("simulations of meiosis and crosses work", {

    # these are all rather contrived, but it's a start

    seed <- 79251223
    set.seed(seed)

    expected <- c(36.1079982714728, 51.9751604413614, 204.360923892818)
    expect_equal(sim_crossovers(300, 10, 0.3), expected)

    allele <- ifelse(runif(1) < 0.5, 1L, 2L)
    expect_equal(allele, 1L)

    another_set <- sim_crossovers(300, 3, 0.01)
    another_allele <- ifelse(runif(1) < 0.5, 1L, 2L)
    expect_equal(another_allele, 2L)

    p1 <- create_parent(300, 1)
    p2 <- create_parent(300, 2)
    f1 <- cross(p1, p2)
    expect_equal(create_parent(300, 1:2), f1)

    set.seed(seed)
    expected2 <- list(alleles=c(1L, 2L, 1L, 2L), locations=c(expected, 300))
    expect_equal(sim_meiosis(f1, m=10, p=0.3), expected2)

    expected3 <- list(alleles=c(2L, 1L, 2L), locations=c(another_set, 300))
    expect_equal(sim_meiosis(f1, m=3, p=0.01), expected3)

    set.seed(seed)
    f2 <- cross(f1, f1, m=10, p=0.3)
    expected <- list(mat=expected2,
                     pat=list(alleles=c(1L, 2L, 1L),
                              locations=c(22.7139970986173, 234.89673093427, 300)))
    expect_equal(f2, expected)

    set.seed(seed)
    junk <- sim_meiosis(f1, m=10, p=0.3)
    f2 <- cross(f1, f1, m=3, p=0.01)
    expected <- list(mat=expected3,
                     pat=list(alleles=c(2L, 1L, 2L, 1L),
                     locations=c(72.8879569564015, 194.954296317883, 285.253091622144, 300)))
    expect_equal(f2, expected)

    set.seed(seed)
    p1 <- create_parent(300, 1:2)
    p2 <- create_parent(300, 3:4)
    f1a <- cross(p1, p2)
    f2a <- cross(p1, p2)
    sib <- cross(f1a, f2a)
    expected <- list(mat=list(alleles=c(4L, 3L, 2L, 3L, 2L, 1L, 4L),
                                    locations=c(113.656340399757,
                                                119.926706166007, 151.954430225305, 188.945353589952,
                                                214.686310663819, 242.297911760397, 300)),
                     pat=list(alleles=c(4L, 3L, 4L, 3L, 4L),
                                    locations=c(130.161284795031, 178.185045230202, 224.519506352954,
                                                250.761620071717, 300)))

    expect_equal(sib, expected)

})


test_that("average no. crossovers match expectation", {

    set.seed(82271836)
    tol <- 0.01
    n <- 10000

    x <- replicate(n, sim_crossovers(L=100, m=0, p=0))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    Lstar <- calc_Lstar(100, 0, 0)
    x <- replicate(n, sim_crossovers(L=100, m=0, p=0, obligate_chiasma=TRUE, Lstar=Lstar))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    x <- replicate(n, sim_crossovers(L=100, m=1, p=0))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    Lstar <- calc_Lstar(100, 1, 0)
    x <- replicate(n, sim_crossovers(L=100, m=1, p=0, obligate_chiasma=TRUE, Lstar=Lstar))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    set.seed(82271836)
    x <- replicate(n, sim_crossovers(L=100, m=10, p=0))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    Lstar <- calc_Lstar(100, 10, 0)
    x <- replicate(n, sim_crossovers(L=100, m=10, p=0, obligate_chiasma=TRUE, Lstar=Lstar))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    x <- replicate(n, sim_crossovers(L=100, m=10, p=0.1))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

    Lstar <- calc_Lstar(100, 10, 0.1)
    x <- replicate(n, sim_crossovers(L=100, m=10, p=0.1, obligate_chiasma=TRUE, Lstar=Lstar))
    expect_equal(mean(sapply(x, length)), 1, tolerance=tol)

})

# get_geno
#
#' Get genotype at a single position
#'
#' With data on the continuous crossover location information produced by
#' sim_from_pedigree, grab the genotype at a given position.
#'
#' @param xodat The sort of detailed genotype/XO data generated by
#' [sim_from_pedigree()]
#' @param position Position (in cM) for which to obtain genotypes
#'
#' @return A numeric matrix with two columns: the maternal and
#' paternal allele for each individual.
#'
#' @export
#' @keywords utilities
#' @seealso [sim_from_pedigree()], [convert2geno()]
#'
#' @examples
#' # simulate AIL pedigree
#' tab <- sim_ail_pedigree(12, 30)
#' # simulate data from that pedigree
#' dat <- sim_from_pedigree(tab)
#' # get genotype at position 30 cM
#' geno <- get_geno(dat, 30)
get_geno <-
    function(xodat, position)
{
    L <- max(xodat[[1]]$mat$locations)

    if(position < 0 || position > L)
        stop("position is outside the range of the data")
    if(position == 0) position <- -1

    tol <- 1e-8
    if(position > L - tol)
        position <- L - tol

    output <- .get_geno(xodat, position)
    dimnames(output) <- list(names(xodat), c("mat", "pat"))

    output
}

context("get_geno")

test_that("simple test of get_geno", {

    dat <- list("1"=list(mat=list(alleles=c(1, 2, 1), locations=c(9.22, 83.6, 100)),
                         pat=list(alleles=c(2, 1), locations=c(48.7, 100))),
                "2"=list(mat=list(alleles=c(2, 1, 2), locations=c(18.64, 55.21, 100)),
                         pat=list(alleles=c(2, 1, 2), locations=c(31.55, 95.42, 100))),
                "3"=list(mat=list(alleles=c(2, 1, 2), locations=c(5.5, 39.4, 100)),
                         pat=list(alleles=c(1, 2),    locations=c(89.1, 100))))

    ex_geno <- rbind(c(2, 2), c(1, 2), c(1, 1))
    geno <- get_geno(dat, 30)
    expect_equivalent(geno, ex_geno)

    ex_geno <- rbind(c(1,2), c(2,2), c(2, 1))
    geno <- get_geno(dat, 0)
    expect_equivalent(geno, ex_geno)

    ex_geno <- rbind(c(1,1), c(2,2), c(2, 2))
    geno <- get_geno(dat, 100)
    expect_equivalent(geno, ex_geno)

    expect_error(get_geno(dat, -1))
    expect_error(get_geno(dat, 101))

})

# sim_4way_pedigree
#
#' Simulate pedigree for 4-way intercross
#'
#' Simulate a 4-way cross, among four inbred lines (a table of
#' individual, mom, dad, sex)
#'
#' @param ngen Number of intercross generations (1 or 2)
#' @param nsibs Vector with number of siblings in the sibships in the
#' last generation.
#'
#' @return A data frame with five columns: individual ID, mother ID,
#' father ID, sex, and generation.  Founders have `0` for mother
#' and father ID. Sex is coded 0 for female and 1 for male.
#'
#' @details We start with a set of 4 individuals (representing four
#' inbred lines), and make a pair of crosses to generate a pair of
#' heterozygous individuals.  These are then crosses to generate a set
#' of F1 individuals. If `ngen==1`, we stop there, with
#' `sum(nsibs)` individuals in this last generation.  If
#' `gen==2`, we generate `length(nsibs)` male/female pairs
#' of F1 offspring; these are intercrossed to generate a set of
#' sibships, with lengths defined by the values in `nsibs`.
#' Individuals in the last generation are alternating female/male.
#'
#' @export
#' @keywords datagen
#' @seealso [sim_from_pedigree()],
#' [sim_ril_pedigree()], [sim_do_pedigree()],
#' [sim_ail_pedigree()]
#'
#' @examples
#' # 100 F1s between heterozygous parents
#' tab <- sim_4way_pedigree(1, 100)
#' # could also do this
#' tab2 <- sim_4way_pedigree(1, rep(10, 10))
#'
#' # 120 F2s in 10 sibships each of size 12
#' tab3 <- sim_4way_pedigree(ngen=2, rep(12, 10))
sim_4way_pedigree <-
    function(ngen=1, nsibs=100)
{
    if(ngen != 1 && ngen != 2)
        stop("ngen must be 1 or 2")
    if(any(is.na(nsibs) | nsibs <= 0))
        stop("nsibs can't have missing or non-positive values")

    # initial generations
    id <-  c(1,2,3,4, 5,6)
    mom <- c(0,0,0,0, 1,3)
    dad <- c(0,0,0,0, 2,4)
    sex <- c(0,1,0,1, 0,1)
    gen <- c(0,0,0,0, 1,1)

    if(ngen==1) nf1 <- sum(nsibs)
    if(ngen==2) nf1 <- 2*length(nsibs)

    id <- c(id, (1:nf1)+6)
    mom <- c(mom, rep(5, nf1))
    dad <- c(dad, rep(6, nf1))
    sex <- c(sex, rep(c(0,1), ceiling(nf1/2))[1:nf1])
    gen <- c(gen, rep(2, nf1))

    if(ngen==2) {
        totsibs <- sum(nsibs)

        prev <- length(id)
        for(i in seq(along=nsibs)) {
            id <- c(id, prev+(1:nsibs[i]))
            mom <- c(mom, rep(i*2+5, nsibs[i]))
            dad <- c(dad, rep(i*2+6, nsibs[i]))
            prev <- length(id)
        }
        sex <- c(sex, rep(c(0,1), ceiling(totsibs/2))[1:totsibs])
        gen <- c(gen, rep(3, totsibs))
    }

    data.frame(id=id, mom=mom, dad=dad, sex=sex, gen=gen)
}

#' Simulate genotypes for pedigree for multiple chromosomes
#'
#' Simulate genotypes along all chromosomes for a pedigree. This is a
#' wrap up of sim_from_pedigree.
#'
#' @inheritParams sim_from_pedigree
#' @param map marker locations, a list with elements for each
#' chromosome
#'
#' @return A list with each component being the result from
#' `sim_from_pedigree`, of length same as `map`.
#'
#' @export
#' @keywords datagen
#' @seealso [check_pedigree()],
#' [sim_ril_pedigree()], [sim_ail_pedigree()]
#' [sim_from_pedigree()]
#'
#' @examples
#' library(qtl)
#' # marker map
#' map <- sim.map(len=rep(100, 19), n.mar=10, include.x=FALSE)
#' # simulate AIL pedigree
#' tab <- sim_ail_pedigree(12, 30)
#' # simulate data from that pedigree
#' dat <- sim_from_pedigree_allchr(tab, map)
#'
sim_from_pedigree_allchr <- function(pedigree, map, m=10, p=0, obligate_chiasma=FALSE){
  chr.len <- sapply(map, max)
  is.xchr <- sapply(map, class) == "X"
  dat <- NULL
  for(chr in 1:length(map))
      dat[[chr]] <- sim_from_pedigree(pedigree, L=chr.len[chr], xchr=is.xchr[chr],
                                      m=m, p=p, obligate_chiasma=obligate_chiasma)
  names(dat) <- names(map)
  return(dat)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.convert2geno <- function(xodat, map, founder_geno) {
    .Call(`_simcross_convert2geno`, xodat, map, founder_geno)
}

convert2genoarray <- function(xodat, map) {
    .Call(`_simcross_convert2genoarray`, xodat, map)
}

convert2geno_char <- function(xodat, map, founder_geno) {
    .Call(`_simcross_convert2geno_char`, xodat, map, founder_geno)
}

convert2geno_char_paste <- function(xodat, map, founder_geno) {
    .Call(`_simcross_convert2geno_char_paste`, xodat, map, founder_geno)
}

.get_geno <- function(xodat, position) {
    .Call(`_simcross_get_geno`, xodat, position)
}

.sim_crossovers <- function(L, m, p, obligate_chiasma, Lstar) {
    .Call(`_simcross_sim_crossovers`, L, m, p, obligate_chiasma, Lstar)
}

.sim_meiosis <- function(parent, m, p, obligate_chiasma, Lstar) {
    .Call(`_simcross_sim_meiosis`, parent, m, p, obligate_chiasma, Lstar)
}

# sim_ril_pedigree
#
#' Generate a ril pedigree
#'
#' Generate a pedigree for multi-way recombinant inbred lines (a table
#' of individual, mom, dad, sex)
#'
#' @param ngen Number of generations of inbreeding
#' @param selfing If TRUE, use selfing
#' @param parents Vector of the parents' IDs. Should be integers, and
#'     length must be a power of 2 (i.e., 2, 4, 8, ...)
#' @param firstind Positive integer to assign to the first child. Must
#'     be greater than `max(parents)`.
#'
#'
#' @return A data frame with five columns: individual ID, mother ID,
#' father ID, sex, and generation.  Founders have `0` for mother
#' and father ID. Sex is coded 0 for female and 1 for male.
#'
#' @export
#' @seealso [sim_from_pedigree()],
#' [sim_ail_pedigree()], [sim_do_pedigree()],
#' [sim_4way_pedigree()]
#' @keywords datagen
#'
#' @examples
#' tab <- sim_ril_pedigree(7)
sim_ril_pedigree <-
    function(ngen=20, selfing=FALSE, parents=1:2, firstind=max(parents)+1)
{
    if(!is.numeric(parents))
        stop("parents should be vector of integers")

    nparents <- length(parents)
    if(nparents %% 2 != 0 || nparents<2)
        stop("number of parents must = 2^k for some positive k")

    if(firstind <= max(parents))
        stop("firstind should be > max(parents); otherwise there will be ID conflicts")

    # first generation of pedigree
    ped <- data.frame(id=parents,
                      mom=0,
                      dad=0,
                      sex=c(0,1),
                      gen=0)
    nextind <- firstind
    generation <- 0

    # parents -> matrix of couples
    parents <- matrix(parents, ncol=2, byrow=TRUE)

    # pedigree up to the last outcross
    while(length(parents) > 2) {
        generation <- generation + 1
        offspring <- seq(nextind, by=1, length=nrow(parents))
        nextind <- max(offspring) + 1
        ped <- rbind(ped,
                     data.frame(id=offspring,
                                mom=parents[,1],
                                dad=parents[,2],
                                sex=c(0,1),
                                gen=generation) )

        parents <- matrix(offspring, ncol=2, byrow=TRUE)
    }

    # parents now has length 2
    generation <- generation + 1

    if(selfing) {
        # one last cross
        ped <- rbind(ped,
                     data.frame(id=nextind,
                                mom=parents[,1],
                                dad=parents[,2],
                                sex=0,
                                gen=generation))
        # then start selfing
        nextind <- nextind + 1
        for(g in generation + 1:ngen) {
            ped <- rbind(ped,
                         data.frame(id=nextind,
                                    mom=nextind-1,
                                    dad=nextind-1,
                                    sex=0,
                                    gen=g))
            nextind <- nextind + 1
        }
    }
    else {
        # one last cross
        ped <- rbind(ped,
                     data.frame(id=c(nextind,nextind+1),
                                mom=parents[,1],
                                dad=parents[,2],
                                sex=c(0,1),
                                gen=generation))
        # then start selfing
        nextind <- nextind + 2
        for(g in generation + 1:ngen) {
            ped <- rbind(ped,
                         data.frame(id=c(nextind,nextind+1),
                                    mom=nextind-2,
                                    dad=nextind-1,
                                    sex=c(0,1),
                                    gen=g))
            nextind <- nextind + 2
        }

    }
    ped
}

context("sim_ril_pedigree")

test_that("result of sim_ril_ped passes check_pedigree", {

    # parents must have length (2, 4, 8)
    expect_error(sim_ril_pedigree(parents=1:6))
    expect_error(sim_ril_pedigree(parents=NULL))
    expect_error(sim_ril_pedigree(parents=1))

    # results should pass check_pedigree
    for(i in c(2,4,8)) {
        expect_true(check_pedigree(sim_ril_pedigree(parents=1:i, selfing=FALSE)))
        expect_true(check_pedigree(sim_ril_pedigree(parents=1:i, selfing=TRUE),
                                   ignore_sex=TRUE))
    }

})

context("check_pedigree")

test_that("simple tests of check_pedigree", {

    tab <- sim_ril_pedigree(7)
    expect_true(check_pedigree(tab))

    # not in correct order
    tab2 <- tab[c(9, 1:8, 10:nrow(tab)),]
    expect_error(check_pedigree(tab2))

    # male mom
    tab3 <- tab
    tab3[1,4] <- 1
    expect_error(check_pedigree(tab3))

    # female dad
    tab4 <- tab
    tab4[2,4] <- 0
    expect_error(check_pedigree(tab4))

})

# sim_from_pedigree
#
#' Simulate genotypes for pedigree
#'
#' Simulate genotypes along one chromosome for a pedigree
#'
#' @param pedigree Matrix or data frame describing a pedigree, with first four
#' columns being individual ID, mom ID, dad ID, and sex (female as
#' `0`, male as `1`).
#' @param L Length of chromosome in cM (or a vector of chromosome lengths)
#' @param xchr If TRUE, simulate X chromosome. (If `L` is a
#' vector, this should be a vector of TRUE/FALSE values, of the same
#' length as `L`, or a character string with the name of the X
#' chromosome, in `L`.)
#' @param m Crossover interference parameter, for chi-square model
#' (m=0 corresponds to no interference).
#' @param p proportion of crossovers coming from no-interference process
#' @param obligate_chiasma If TRUE, require an obligate chiasma on the
#' 4-strand bundle at meiosis.
#'
#' @return A list with each component being the data for one
#' individual, as produced by the [cross()] function.  Those
#' results are a list with two components, corresponding to the
#' maternal and paternal chromosomes. The chromosomes are represented
#' as lists with two components: an integer vector of alleles in
#' chromosome intervals, and a numeric vector of locations of the
#' right-endpoints of those intervals; these two vectors should have
#' the same length.
#'
#' If the input `L` is a vector, in order to simulate multiple
#' chromosomes at once, then the output will be a list with length
#' `length(L)`, each component being a chromosome and having the
#' form described above.
#'
#' @export
#' @keywords datagen
#' @seealso [check_pedigree()],
#' [sim_ril_pedigree()], [sim_ail_pedigree()],
#' [sim_from_pedigree_allchr()]
#'
#' @examples
#' # simulate AIL pedigree
#' tab <- sim_ail_pedigree(12, 30)
#' # simulate data from that pedigree
#' dat <- sim_from_pedigree(tab)
#' # simulate multiple chromosomes
#' dat <- sim_from_pedigree(tab, c("1"=100, "2"=75, "X"=100), xchr="X")
sim_from_pedigree <-
    function(pedigree, L=100, xchr=FALSE, m=10, p=0, obligate_chiasma=FALSE)
{
    if(length(L) > 1) { # multiple chromosomes
        result <- vector("list", length(L))
        if(is.null(names(L))) names(L) <- seq(along=L)
        names(result) <- names(L)

        if(is.character(xchr)) # xchr is chromosome names
            xchr <- names(L) %in% xchr

        if(is.null(xchr))
            xchr <- rep(FALSE, length(L))

        if(length(xchr) == 1) # if single value, apply to all chromosomes
            xchr <- rep(xchr, length(L))

        if(length(xchr) != length(L))
            stop("length(xchr) != length(L)")

        for(i in seq(along=L))
            result[[i]] <- sim_from_pedigree(pedigree, L[i], xchr[i],
                                             m, p, obligate_chiasma)
        return(result)
    }

    if(length(unique(pedigree[,1])) != nrow(pedigree))
        stop("IDs must be unique")
    rownames(pedigree) <- pedigree[,1]

    result <- vector("list", nrow(pedigree))
    names(result) <- as.character(pedigree[,1])

    if(obligate_chiasma) Lstar <- calc_Lstar(L, m, p)
    else Lstar <- L

    for(i in 1:nrow(pedigree)) {
        if(pedigree[i,2]==0 || pedigree[i,3]==0) # founder
            result[[i]] <- create_parent(L, allele=pedigree[i,1])
        else {
            mom <- which(pedigree[,1]==pedigree[i,2])
            dad <- which(pedigree[,1]==pedigree[i,3])

            if(length(mom) < 1 || length(dad) < 1) {
                print(pedigree[i,,drop=FALSE])
                stop("parents not found")
            }
            if(mom >= i || dad >= i) {
                print(pedigree[i,,drop=FALSE])
                stop("Pedigree problem: parents follow individual")
            }

            result[[i]] <- cross(result[[mom]], result[[dad]],
                                 m=m, p=p,
                                 xchr=xchr, male=pedigree[i,4]==1,
                                 obligate_chiasma=obligate_chiasma,
                                 Lstar=Lstar)
        }
    }
    result
}

# collapse_do_alleles
#' Collapse alleles for simulated DO genotypes
#'
#' When simulating Diversity Outbreds, we need to specify parents
#' 1-16, with 9-16 being the males from strains 1-8.  This function
#' collapses replaces alleles 9-16 with 1-8, to make the result
#' ordinary DO-type data.
#'
#' @param xodata The sort of detailed genotype/crossover data
#' generated by [sim_from_pedigree()].
#'
#' @return The input object, with alleles 9-16 replaced by 1-8.
#'
#' @export
#' @keywords utilities
#' @seealso [sim_do_pedigree()], [sim_do_pedigree_fix_n()],
#' [sim_from_pedigree()]
#'
#' @examples
#' # simulate DO pedigree
#' tab <- sim_do_pedigree(8)
#'
#' # simulate genotypes for that pedigree
#' dat <- sim_from_pedigree(tab)
#' # collapse to alleles 1-8
#' dat <- collapse_do_alleles(dat)
#'
#' # also works with data on multiple chromosomes
#' \donttest{dat <- sim_from_pedigree(tab, c("1"=100, "2"=75, "X"=100), xchr="X")
#' dat <- collapse_do_alleles(dat)}

collapse_do_alleles <-
    function(xodata)
{
    if(!("mat" %in% names(xodata[[1]]))) { # multiple chromosomes?
        if(!("mat" %in% names(xodata[[1]][[1]])))
            stop("input doesn't look like the output of sim_from_pedigree")
        return(lapply(xodata, collapse_do_alleles))
    }

    lapply(xodata, function(a) {
        # grab alleles
        mata <- a$mat$alleles
        pata <- a$pat$alleles
        matl <- a$mat$locations
        patl <- a$pat$locations

        # 9-16 -> 1-8
        mata[mata > 8] <- mata[mata > 8] - 8
        pata[pata > 8] <- pata[pata > 8] - 8

        if(any(diff(mata)==0)) { # adjacent alleles the same
            drop <- which(diff(mata)==0)
            mata <- mata[-(drop+1)]
            matl <- matl[-drop]
        }
        if(any(diff(pata)==0)) { # ajacent alleles the same
            drop <- which(diff(pata)==0)
            pata <- pata[-(drop+1)]
            patl <- patl[-drop]
        }

        # paste back into object
        a$mat$alleles <- mata
        a$pat$alleles <- pata
        a$mat$locations <- matl
        a$pat$locations <- patl

        a })

}

#' Mouse chromosome lengths
#'
#' Mouse chromosome lengths in cM from the Mouse Genome Informatics
#' (MGI) standard map.
#'
#' @docType data
#'
#' @usage data(mouseL_mgi)
#'
#' @format A numeric vector with lengths in cM for the 20 mouse chromosomes.
#'
#' @source Taken from Table 1 of Cox et al. (2009) A new standard
#'     genetic map for the laboratory mouse. Genetics 182:1335-1344.
#'     \doi{10.1534/genetics.109.105486}
#'
#' @keywords datasets
#'
#' @seealso mouseL_cox
#'
#' @examples
#' data(mouseL_mgi)
"mouseL_mgi"

# sim_dof1_pedigree
#
#' Simulate pedigree for F1 between diversity outbreds and another inbred line
#'
#' Simulate a pedigree for a set of DOF1 individuals: the F1 offspring
#' of a set of diversity outbred mice and another inbred strain (such
#' as a mutant line).
#'
#' @param ngen Number of generations of outbreeding
#' @param npairs Number of breeding pairs at each generation
#' @param ccgen Vector of length `npairs`, with the number of
#' generations for each CC line. If length 1, it is repeated to a
#' vector of length `npairs`.
#' @param nkids_per Number of offspring per pair for the last
#' DO generation (each will be crossed to produce one F1)
#' @param design How to choose crosses: either random but avoiding
#' siblings, or completely at random
#'
#' @return A data frame with seven columns: individual ID, mother ID,
#' father ID, sex, generation, a TRUE/FALSE indicator for whether DO
#' or pre-DO, and a TRUE/FALSE indicator for whether DOF1.  Founders
#' have `0` for mother and father ID. Sex is coded 0 for female
#' and 1 for male.
#'
#' @details Diversity outbred (DO) mice are generated from a set of 8 inbred
#' lines. We need two individuals from each line (one female and one
#' male) as the order of the initial crosses will be randomized; for
#' example, sometimes the individual from line 1 will be a mother and
#' sometimes a father. The founders are numbered 1-8 for the females
#' from the 8 lines, and 9-16 for the corresponding males.
#'
#' Diversity Outbred mice are generated by first creating a panel of
#' partially-inbred 8-way RIL (the so-called pre-CC, for
#' pre-Collaborative Cross). The `ccgen` argument specifies the
#' number of inbreeding generations for each of the CC lines. We
#' generate a pre-CC line for each of the `npairs` breeding
#' pairs, and generate a sibling pair from each as the starting
#' material.
#'
#' The subsequent `ngen` outbreeding generations then proceed by
#' crossing a male and female from the preceding generation (mated
#' completely at random, with `design="random"`, or avoiding
#' siblings, with `design="nosib"`). Each breeding pair gives a
#' single female and a single male to the next generation, except at
#' the last generation `nkids_per` offspring are mated, in equal
#' numbers male and female. (If `nkids_per` is an odd number, the
#' number of males and females in each sibship will differ by one,
#' alternating between sibships, with one additional female and then
#' one additional male.
#'
#' The default for `ccgen` is taken from Figure 1 of Svenson et
#' al. (2012).
#'
#' We assume that the F1 offspring are all from a cross DO female x
#' line 17 male, and so the last generation of the DO is taken to be
#' all females.
#'
#' @references
#' Svenson KL, Gatti DM, Valdar W, Welsh CE, Cheng R, Chesler EJ,
#' Palmer AA, McMillan L, Churchill GA (2012) High-resolution genetic
#' mapping using the mouse Diversity Outbred population. Genetics
#' 190:437-447
#'
#' @keywords datagen
#' @export
#' @seealso [sim_from_pedigree()],
#' [sim_ril_pedigree()], [sim_ail_pedigree()],
#' [sim_4way_pedigree()]
#'
#' @examples
#' tab <- sim_dof1_pedigree(8)
sim_dof1_pedigree <-
    function(ngen=12, npairs=144,
             ccgen=rep(4:12, c(21, 64, 24, 10, 5, 9, 5, 3, 3)),
             nkids_per=5, design=c("nosib", "random"))
{
    design <- match.arg(design)
    if(length(ccgen)==1) ccgen <- rep(ccgen, npairs)
    stopifnot(length(ccgen) == npairs)
    stopifnot(all(ccgen >= 0))

    # initial generation : need to double each strain
    #     1-8 are the mothers; 9-16 are the corresponding dads
    id <- 1:16
    mom <- rep(0, 16)
    dad <- rep(0, 16)
    sex <- rep(0:1, each=8)
    gen <- rep(0, 16)
    result <- data.frame(id=id, mom=mom, dad=dad, sex=sex, gen=gen)

    cur_nind <- 16
    lastgen <- NULL
    for(i in 1:npairs) {
        # random cross among the 8 strains
        parents <- sample(1:8)
        parents[seq(2, 8, by=2)] <- parents[seq(2, 8, by=2)] + 8

        tab <- sim_ril_pedigree(ccgen[i], selfing=FALSE, parents=parents, firstind=cur_nind+1)[-(1:8),]
        result <- rbind(result, tab)
        lastgen <- c(lastgen, tab[tab[,"gen"] == max(tab[,"gen"]),1])
        cur_nind <- max(tab[,1])
    }
    result <- data.frame(result, do=rep(FALSE, nrow(result)))

    moms <- lastgen[seq(1, length(lastgen), by=2)]
    dads <- lastgen[seq(2, length(lastgen), by=2)]
    for(i in 1:ngen) {
        dads <- sample(dads)
        while(design=="nosib" && any(dads - moms == 1)) { # sample until no sibs
            dads <- sample(dads)
        }

        if(i < ngen) {
            id <- 1:(npairs*2) + cur_nind

            mom <- rep(moms, each=2)
            dad <- rep(dads, each=2)
            sex <- rep(c(0,1), npairs)
            gen <- rep(i, npairs*2)

            moms <- id[seq(1, length(id), by=2)]
            dads <- id[seq(2, length(id), by=2)]
        }
        else { # last generation: expand
            id <- 1:(npairs*nkids_per)+cur_nind

            mom <- rep(moms, each=nkids_per)
            dad <- rep(dads, each=nkids_per)
            sex <- rep(c(0,1), ceiling(npairs*nkids_per/2))[1:(npairs*nkids_per)]
            gen <- rep(i, npairs*nkids_per)
        }
        cur_nind <- max(id)

        newrows <- data.frame(id=id, mom=mom, dad=dad, sex=sex, gen=gen,
                              do=rep(TRUE, length(id)))
        result <- rbind(result, newrows)
    }

    offspring <- result[-(1:16),]
    offspring$id <- offspring$id + 1
    offspring$mom[offspring$mom > 16]  <- offspring$mom[offspring$mom > 16] + 1
    offspring$dad[offspring$dad > 16]  <- offspring$dad[offspring$dad > 16] + 1
    offspring$sex[offspring$do & offspring$gen==ngen] <- 0

    result <- rbind(result[1:16,],
                    data.frame(id=17, mom=0, dad=0, sex=1, gen=0, do=FALSE),
                    offspring)

    last_gen <- result$id[result$do & result$gen==ngen]
    n_last_gen <- length(last_gen)

    result <- rbind(cbind(result, dof1=FALSE),
                    data.frame(id=max(last_gen)+seq(along=last_gen),
                               mom=last_gen,
                               dad=17,
                               sex=sample(0:1, n_last_gen, replace=TRUE),
                               gen=ngen+1,
                               do=FALSE,
                               dof1=TRUE))
    rownames(result) <- 1:nrow(result)
    result
}

# check_pedigree
#
#' Check a pedigree for errors
#'
#' Perform a series of checks on the tabular data for a pedigree,
#' checking for problems
#'
#' @param pedigree Numeric matrix or data frame with four columns: ID,
#' mom ID, dad ID, sex. Sex is coded as `0`=female,
#' `1`=male. There can be additional columns, but they'll be
#' ignored.
#' @param ignore_sex If TRUE, ignore the sex values completely
#' (appropriate for hermaphroditic species.)
#'
#' @return TRUE (invisibly) if everything is okay; otherwise gives an
#' error.
#'
#' @details The parents should be listed before any of their
#' offspring. Founders should have 0's for mother and father; all
#' others should have non-zero values for the parents, and the parents
#' should appear in the pedigree. Father should be male and mothers
#' should be female (unless `ignore_sex=TRUE`). Individual
#' identifiers should be unique and non-zero. There should be no
#' missing values anywhere. (`NA`s are allowed in the sex column
#' if `ignore_sex=TRUE`.)
#'
#' @export
#' @keywords utilities
#' @seealso [sim_from_pedigree()],
#' [sim_ril_pedigree()]
#'
#' @examples
#' tab <- sim_ril_pedigree(7)
#' check_pedigree(tab)
#'
check_pedigree <-
    function(pedigree, ignore_sex=FALSE)
{
    if(ncol(pedigree) < 4)
        stop("pedigree must have at least 4 columns")
    id <- pedigree[,1]
    mom <- pedigree[,2]
    dad <- pedigree[,3]
    sex <- pedigree[,4]

    # check for missing values
    if(any(is.na(id)))
        stop("missing values not allowed in id column")
    if(any(is.na(mom)))
        stop("missing values not allowed in mom column")
    if(any(is.na(dad)))
        stop("missing values not allowed in dad column")
    if(!ignore_sex && any(is.na(sex)))
        stop("missing values not allowed in sex column")

    # check numeric IDs: non-zero, unique
    if(any(id==0))
        stop("0 is not a valid ID")
    if(length(id) != length(unique(id)))
        stop("IDs must be unique")

    # mom and dad are both 0 or neither 0
    if(any((mom==0 & dad != 0) | (dad==0 & mom != 0)))
        stop("mom and dad must be both 0 or neither 0")

    # parents in pedigree
    match_mom <- match(mom, id)
    match_dad <- match(dad, id)
    if(any(mom!=0 & is.na(match_mom))) {
        not_found <- mom[mom!=0 & is.na(match_mom)]
        stop("Some moms not found: ", paste(not_found, collapse=" "))
    }
    if(any(dad!=0 & is.na(match_dad))) {
        not_found <- dad[dad!=0 & is.na(match_dad)]
        stop("Some dads not found: ", paste(not_found, collapse=" "))
    }
    stopifnot(dad==0 | !is.na(match(dad, id)))

    # parents precede offspring
    match_mom <- match_mom[mom != 0]
    momsub <- which(mom != 0)
    if(any(match_mom >= momsub)) {
        problems <- momsub[match_mom >= momsub]
        stop("Some moms appear after their offspring: ",
             paste(problems, collapse=" "))
    }
    match_dad <- match_dad[dad != 0]
    dadsub <- which(dad != 0)
    if(any(match_dad >= dadsub)) {
        problems <- dadsub[match_dad >= dadsub]
        stop("Some dads appear after their offspring: ",
             paste(problems, collapse=" "))
    }

    if(ignore_sex) return(TRUE) # skip the rest

    # check sex
    if(any(sex[match_mom] != 0)) {
        male_moms <- unique(pedigree$mom[momsub[sex[match_mom] != 0]])
        stop("Some moms are male: ",
             paste(male_moms, collapse=" "))
    }
    if(any(sex[match_dad] != 1)) {
        female_dads <- unique(pedigree$dad[dadsub[sex[match_dad] != 1]])
        stop("Some dads are female: ",
             paste(female_dads, collapse=" "))
    }

    TRUE
}

# create a dataset from the AIL pedigree in the QTLRel package

library(QTLRel)
data(miscEx)
AILped <- cbind(id=pedF8$id,
                mom=pedF8$dam,
                dad=pedF8$sire,
                sex=ifelse(pedF8$sex=="M", 1, 0),
                generation=as.numeric(substr(pedF8$generation, 2, 2)))

# renumber individuals
newid <- c(0,order(AILped[,"id"]))
names(newid) <- c("0", as.character(AILped[,"id"]))
for(i in 1:3)
    AILped[,i] <- newid[as.character(AILped[,i])]

# check that the change in sex is correct
newsex <- factor(c("F", "M")[AILped[,"sex"]+1], levels=c("F", "M"))
library(testthat)
expect_equal(pedF8$sex, newsex)

# check that the renumbering is correct
mom <- match(AILped[,2], AILped[,1])
momo <- match(pedF8$dam, pedF8$id)
expect_equal(mom, momo)

dad <- match(AILped[,3], AILped[,1])
dado <- match(pedF8$sire, pedF8$id)
expect_equal(dad, dado)

# generation the same
gen <- factor(paste0("F", AILped[,"generation"]))
expect_equal(gen, pedF8$generation)

# save to file
AILped <- as.data.frame(AILped)
save(AILped, file="../../data/AILped.RData")

---
title: R/simcross User Guide
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{R/simcross User Guide}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8](inputenc)
---

```{r knitr_options, include=FALSE}
library(knitr)
opts_chunk$set(fig.width=7, fig.height=4.5,
               dev.args=list(pointsize=16))
```

<link rel=stylesheet type="text/css" href="https://kbroman.org/stylesheets/kbroman_vignette.css"></script>

<a href="https://kbroman.org/simcross/"><img src="https://kbroman.org/simcross/assets/pics/simcross_logo.png" align="right" height="138" alt="R/simcross logo"/></a>

[![R-CMD-check](https://github.com/kbroman/simcross/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kbroman/simcross/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/simcross)](https://cran.r-project.org/package=simcross)
[![r-universe badge](https://kbroman.r-universe.dev/simcross/badges/version)](https://kbroman.r-universe.dev/simcross)
[![zenodo DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.4032914.svg)](https://doi.org/10.5281/zenodo.4032914)

[R/simcross](https://kbroman.org/simcross/) is an
[R](https://www.r-project.org) package to simulate genotypes from an
experimental cross. The aim is flexibility rather than speed.

Meiosis is simulated following the Stahl model (Copenhaver et
al. 2002) with the interference parameter being an integer. In this
model, chiasma locations are the superposition of two processes: a
proportion _p_ come from a process exhibiting no interference (that
is, a Poisson process) and the remainder (proportion 1 &ndash; _p_)
come from a process following the chi-square model (Foss et al. 1993;
Zhao et al. 1995). Thus, with _p_=0, the model reduces to the
chi-square model.  The chi-square model has a single parameter, _m_,
which is a non-negative integer and controls the strength of
interference. _m_=0 corresponds to no interference.
[Broman et al. (2002)](https://pubmed.ncbi.nlm.nih.gov/11901128/) estimated _m_=10 for the level of interference in the mouse
(derived assuming _p_=0).  The chi-square model is a special case of
the gamma model (McPeek and Speed 1995) which has a positive parameter
_&nu;_; the chi-square model corresponds to the case that _m_ = _&nu;_
&ndash; 1 is an integer.

In many organisms, there is always at least one chiasma for each pair
of homologous chromosomes at meiosis. Simulations with
[R/simcross](https://kbroman.org/simcross/) may be performed
with this assumption; the same model is used, but with rejection
sampling to get results conditional on there being at least one
chiasma. Note that this isn't an assumption of an obligate
_crossover_; if there is exactly one chiasma, then a random meiotic
product will have 0 or 1 crossovers, with probability 1/2 each. Also
note that, with the obligate chiasma assumption, chromosomes must be
greater than 50 cM.

## Basic functions

There are two basic functions for simulating a cross:
`create_parent` and `cross`.

`create_parent` is for generating a parent object, either an
inbred individual or the F<sub>1</sub> offspring of two inbred
individuals. It takes two arguments: the length of the chromosome in
cM, and the allele or pair of alleles.

Here's how to generate two inbred individuals, one from each of strain
1 and strain 2, and an F<sub>1</sub> individual, with a 100 cM
chromosome.

```{r set.seed, include=FALSE}
set.seed(80607574)
```

```{r create_parent}
library(simcross)
p1 <- create_parent(L=100, allele=1)
p2 <- create_parent(L=100, allele=2)
f1 <- create_parent(L=100, allele=1:2)
```

You don't need to specify the arguments by name (e.g., in the last
line above, you could just write `create_parent(100, 1:2)`); I'm doing so
here just to better document the names of the arguments.

The `cross` function is used to generate one offspring from the cross
between two individuals.  The input is a pair of individuals (e.g., as
produced by `create_parent`), the interference parameter `m` (default
`m=10`), the parameter `p` for the Stahl model (default `p=0`), `xchr`
to indicate whether the chromosome being simulated is the X chromosome
(`FALSE`, the default, to simulate an autosome), and `male` to
indicate whether the offspring is to be male (which only matters if
`xchr=TRUE`). Further, one may use `obligate_chiasma=TRUE` (default
`FALSE`) to require at least one chiasma on the 4-strand bundle).

Here's an example, to generate an F<sub>2</sub> individual.  (The
outer parentheses cause the result to be printed.)

```{r sim_f2}
(f2 <- cross(f1, f1))
```

The output is a list with two components: the maternal and paternal
chromosomes. Each chromosome is a list with the allele in a set
of intervals, and the locations of the right endpoints of the
intervals.

In the example above, the maternal chromosome is a non-recombinant `2`
chromosome. The paternal chromosome has two crossovers, with the
the `1` allele up to `r round(f2$pat$locations[1], 1)` cM, the `2` allele
in the interval `r round(f2$pat$locations[1], 1)` &ndash;
`r round(f2$pat$locations[2], 1)`, and the `1` allele
for the remainder of the chromosome.

By default, we use `m=10`, `p=0`, and `obligate_chiasma=FALSE`. The
chromosome length is taken from the input parent objects (`f1` in this case).
If we wanted to do the simulation with no crossover interference, but
with an obligate chiasma, we'd use:

```{r sim_f2_nointerference}
f2 <- cross(f1, f1, m=0, obligate_chiasma=TRUE)
```

Behind the scenes, there are two additional
functions, `sim_crossovers`, for simulating crossover locations on a
chromosome, and `sim_meiosis`, for simulating a meiotic product from
an individual, but in general the user need not bother with these.
The `cross` function calls `sim_meiosis` twice
(once for each parent) and then combines the results into a single
individual. `sim_meiosis` calls `sim_crossovers` to generate the
meiotic product.

## Generating pedigrees

While one could simulate any experimental cross from a series of calls
to `create_parent` and `cross`, it is generally more efficient to first develop
a table that describes pedigree for the cross, and then simulate from
the pedigree with the function `sim_from_pedigree` (see the next
section).

### Example AIL pedigree matrix

To define a pedigree, we use a numeric matrix (or data frame) with four columns:
individual ID, mom, dad, and sex (coded as 0=female, 1=male). The
[R/simcross package](https://kbroman.org/simcross/) includes a
sample pedigree for advanced intercross lines, `AILped`, taken from
the [QTLRel package](https://cran.r-project.org/package=QTLRel).
Here is the top of that dataset:

```{r AILped_head}
head(AILped, n=10)
```

### Checking a pedigree matrix

The function `check_pedigree` can be used to check that a pedigree
matrix conforms to R/simcross's requirements: founders have
`mom == dad == 0`, all other individuals have both parents
present in the pedigree, and parents always precede any of their
children. The `check_pedigree` function returns `TRUE` if the pedigree
matrix is okay; otherwise, it throws an error.

```{r check_AILped}
check_pedigree(AILped)
```

R/simcross includes a set of functions for generating pedigree
matrices for different cross designs: `sim_ril_pedigree`,
`sim_4way_pedigree`, `sim_ail_pedigree`, and `sim_do_pedigree`.

### RIL pedigree

`sim_ril_pedigree` generates a pedigree matrix for a single
recombinant inbred line derived from 2^k^ founder
lines for some k>0. (Examples include the Collaborative Cross (Threadgill and
Churchill 2012) and MAGIC lines (Kover et al. 2009).)  The arguments
are `ngen` (number of generations of inbreeding), `selfing` (default
`FALSE`, for sibling mating), `parents` (a vector of integers for the
parents; the length must be a power of 2 (i.e., 2, 4, 8, 16, etc.) and corresponds to the number
of founder lines), and `firstind`, the ID number to attach to the
first individual following the parents (so that the pedigree matrices
for multiple RIL may be `rbind`-ed together). Note that there's no real
_simulation_ here; the result is entirely deterministic.)

```{r sim_ril_ped}
(ril <- sim_ril_pedigree(ngen=4, parents=1:4))
```

The `gen` column in the output is the generation number, with 0
corresponding to the founders. The generations are simply sequential
and so don't correspond to the numbering scheme used for the
Collaborative Cross (see, for example, [Broman 2012](https://pubmed.ncbi.nlm.nih.gov/22345609/)).

### 4-way cross pedigree

`sim_4way_pedigree` generates a pedigree matrix for an intercross
among four inbred lines. The arguments are `ngen` (which must be 1 or
2) and `nsibs`. We start with four inbred individuals, and cross them
in two pairs to generate a pair of heterozygous individuals. If
`ngen==1`, we then just generate a set of `sum(nsibs)` F<sub>1</sub>
offspring. If `ngen==2`, we generate `length(nsibs)` pairs of
F<sub>1</sub>s and intercross them to generate a set of F<sub>2</sub>
sibships; in this case, the input vector `nsibs` determines the sizes
of the sibships.

The following generates two F<sub>2</sub> sibships with 3
offspring in each.

```{r sim_4way_ped}
(fourway <- sim_4way_pedigree(ngen=2, nsibs=c(3, 3)))
```

### AIL pedigree

Advanced intercross lines (AIL) are generated by crossing two inbred
lines to form the F<sub>1</sub> hybrid, intercrossing to form the
F<sub>2</sub> generation, and then performing repeated intercrosses
with some large set of breeding pairs. At each generation, the mating
pairs are chosen at random, often with an effort to avoid matings
between sibling pairs. I would prefer the term &ldquo;advanced
intercross _populations_,&rdquo; as it's a set of heterozygous,
genetically distinct individuals; they aren't really _lines_.

`sim_ail_pedigree` generates a pedigree matrix for 2-way advanced
intercross lines. Unlike `sim_ril_pedigree` and `sim_4way_pedigree`,
this is actually a simulation, as the mating pairs at each generation
are chosen at random. The arguments are `ngen` (number of
generations), `npairs` (number of mating pairs at each generation),
`nkids_per` (for number of kids per sibship in the last generation),
and `design` (`"nosib"` to avoid matings between siblings, or
`"random"` to form the matings completely at random).  At each
generation, each mating pair gives two offspring (one male and one
female) for the next generation. At the last generation, there are
`nkids_per` offspring per mating pair, to give a total of
`npairs*nkids_per` at the last generation.

Here's an example of the use of `sim_ail_pedigree`:

```{r sim_ail_ped}
ailped <- sim_ail_pedigree(ngen=12, npairs=100, nkids_per=5)
nrow(ailped)
table(ailped$gen)
```

With 100 breeding pairs and 5 kids per pair in last generation, we
have 200 individuals for most generations and 500 at the end.

### Diversity Outbred pedigree

The Diversity Outbred population (DO; Svenson et al. 2013) is like AIL, but
starting with eight inbred lines rather than two. Actually, the mouse
DO started with partially-inbred individuals from Collaborative Cross
lines (the so-called preCC; intermediate generations in the
development of eight-way RIL). Heterogeneous stock (HS; Mott and Flint
2002) can be viewed as a special case, but starting with the eight
founder lines.

`sim_do_pedigree` generates a pedigree matrix for a DO population.
The arguments are just like those of `sim_ail_pedigree`, but with one
addition: `ccgen`, which is a vector with the numbers of generations
of inbreeding to form the initial preCC lines that are then used to
initiate the DO. (The default for `ccgen` is taken from Figure 1 of
Svenson et al. 2013.) Use `ccgen=0` to simulate a pedigree for HS.
The length `ccgen` should be `npairs`, the number of breeding pairs;
we take two individuals (one female and one male) from each preCC line
to begin the outcrossing generations.

```{r sim_do_ped}
doped <- sim_do_pedigree(ngen=12)
nrow(doped)
table(do=doped$do, gen=doped$gen)
```

The output contains the usual `id`, `mom`, `dad`, and `sex`, plus
`gen` (for generation number) and `do` (1 indicates part of the DO
population, 0 indicates part of the earlier generations). As you can
see from the table above, the generation (`gen`) has a different
meaning, according to whether `do` is 0 or 1. When `do==0`, it is the
number of generations following the initial founder lines; when
`do==1`, it is the generation number of the outbreeding DO population.

Also note that we start with _sixteen_ lines rather than 8: to
properly handle the X chromosome, we need to consider a male and
female from each of the eight founder lines. These are numbered 1&ndash;8
for the females, and 9&dash;16 for the males. Also note that the preCC
lines are formed from a cross among the eight founders, with the order
of the crosses chosen at random (four females and four males, one from
each of the eight founder lines).

## Simulating from a pedigree

The point of the construction of a pedigree matrix for a cross design,
as in the previous section, was in order to simulate genotype data for
the cross design. The R/simcross function for this is
`sim_from_pedigree`. Its arguments are `pedigree` (the pedigree
matrix), `L` (length of chromosome), `xchr` (`TRUE` or `FALSE`, according to
whether to simulate the X chromosome or an autosome; default `FALSE`),
and then the parameters governing the crossing over process: `m`, `p`,
and `obligate_chiasma`, with defaults `m=10`, `p=0`, and
`obligate_chiasma=FALSE`.

(`L` can also be a vector of chromosome lengths, for simulating
multiple chromosomes at once. In this case `xchr` should be either a
logical vector, of the same length as `L`, or a character string with
the name of the chromosome in `L` that correspond to the X
chromosome. An example of simulating multiple chromosomes is in the
final section in this vignette.)

The `sim_from_pedigree` function calls `create_parent` for all
founding individuals and `cross` for any offspring; the output is a
list with each component corresponding to one individual and having
the form output by `cross`: a list with `mat` and `pat`, for the
maternal and paternal chromosomes, respectively, each of which has
`alleles` (allele present in each interval) and `locations` (location
of right endpoint of each interval).

Here's an example, for simulating an AIL to generation 8.

```{r sim_ail_fully}
ailped <- sim_ail_pedigree(ngen=8, npairs=30, nkids_per=5)
xodat <- sim_from_pedigree(ailped, L=100)
```

Here's the result for the last individual:

```{r last_ail_individual}
xodat[[length(xodat)]]
```

We can plot the average number of breakpoints across the
individuals' two chromosomes, by generation, as follows.

```{r plot_ave_breakpoints}
n_breakpoints <- sapply(xodat, function(a) sum(sapply(a, function(b) length(b$alleles)-1)))
ave_breakpoints <- tapply(n_breakpoints, ailped$gen, mean)
gen <- as.numeric(names(ave_breakpoints))
plot(gen, ave_breakpoints,
     xlab="Generation", ylab="Average no. breakpoints", las=1,
     pch=21, bg="Orchid", main="AIL with 30 breeding pairs")
```

The function `where_het` will show the regions where an individual is
heterozygous.

```{r where_het}
where_het(xodat[[length(xodat)]])
```

We can plot the average proportion of the chromosome that is
heterozygous, by generation, as follows.

```{r plot_prop_het}
prop_het <- sapply(lapply(xodat, where_het), function(a) sum(a[,2]-a[,1])/100)
ave_prop_het <- tapply(prop_het, ailped$gen, mean)
plot(gen, ave_prop_het,
     xlab="Generation", ylab="Average proportion heterozygous", las=1,
     pch=21, bg="Orchid", main="AIL with 30 breeding pairs")
abline(h=0.5, lty=2)
```

Note: if we'd greatly restricted the number of breeding pairs per
generation, we'd see evidence of inbreeding, as a reduced proportion
of heterozygosity. There's considerably more noise, though, since
we've got just 6 individuals per generation.

```{r reset_seed, include=FALSE}
set.seed(28998542)
```

```{r ail_few_pairs}
ailped2 <- sim_ail_pedigree(ngen=8, npairs=3, nkids_per=50)
xodat2 <- sim_from_pedigree(ailped2, L=100)
prop_het2 <- sapply(lapply(xodat2, where_het), function(a) sum(a[,2]-a[,1])/100)
ave_prop_het2 <- tapply(prop_het2, ailped2$gen, mean)
gen2 <- as.numeric(names(ave_prop_het2))
plot(gen2, ave_prop_het2,
     xlab="Generation", ylab="Average proportion heterozygous", las=1,
     pch=21, bg="Orchid", main="AIL with 3 breeding pairs")
abline(h=0.5, lty=2)
```

## Generating marker data

R/simcross simulates the locations of crossovers as continuous values
in the interval (0,L). This is precise and compact, and it allows detailed study
of the breakpoint process, but it can be cumbersome to work with and
is often not what you want from simulations. In most cases, one is
interested in individuals' genotypes at a set of markers.

There are two functions for getting marker genotypes on the basis of
the the detailed crossover location data: `get_geno` and
`convert2geno`.

The function `get_geno` will grab the genotype at a specified location
on the chromosome, returning a matrix with two columns: the maternal
and paternal alleles for each individual. Continuing with the
simulation in the previous section (an AIL with 30 breeding pairs),
here's how to grab the genotype at 30 cM:

```{r get_geno}
g30 <- get_geno(xodat, 30)
tail(g30)
```

The `get_geno` function could be useful, for example, for grabbing QTL
genotypes for use in constructing a phenotype.

The `convert2geno` function takes the detailed crossover information
plus a vector of marker locations and returns a matrix with marker
genotypes.

First, construct a vector with the marker locations.

```{r construct_map}
map <- seq(0, 100, by=10)
names(map) <- paste0("m", map)
map
```

Then, pass the crossover information and map to `convert2geno`. I
print the data for the last five individuals.

```{r convert2geno}
geno <- convert2geno(xodat, map)
geno[nrow(geno)-(4:0), ]
```

Here the genotypes get recoded as `1` / `2` / `3` for `11` / `12` / `22`.
That is, genotypes `1` and `3` are the homozygotes for the allele from
founders 1 and 2, respectively, and genotype `2` is the heterozygote.

For crosses with more than two founders, the output of `convert2geno`
is a three-dimensional array, individuals &times; markers &times;
alleles (maternal and paternal). For example, here are genotypes
for the sixth generations of inbreeding of an eight-way RIL.

```{r convert2geno_8wayril}
rilped <- sim_ril_pedigree(ngen=6, parents=1:8)
dat <- sim_from_pedigree(rilped, L=100)
geno <- convert2geno(dat, map)
geno[nrow(geno)-(1:0),,]
```

More commonly, one may be interested in individuals' SNP
genotypes. This can also be obtained with `convert2geno`; one just
needs to provide a matrix of SNP alleles for the founder lines, with
the argument `founder_geno`. This should be a matrix of `1`s and `2`s,
of dimension `n_founders` &times; `n_markers`.

Let's simulate SNP alleles for eight founder lines.

```{r sim_founder_alleles}
fg <- matrix(sample(1:2, 8*length(map), replace=TRUE), nrow=8)
```

And then here are the SNP genotypes for the sixth generation of
inbreeding an eight-way RIL.

```{r convert2geno_8wayril_snps}
snpgeno <- convert2geno(dat, map, fg)
snpgeno[nrow(snpgeno)-(1:0),]
```

The genotypes are again coded as `1` / `2` / `3` for `11` / `12` /
`22`.


## Simulating multiple chromosomes


One can use `sim_from_pedigree` and `convert2geno` to simulate
multiple chromosomes at once.

To simulate multiple chromosomes with `sim_from_pedigree`,
provide a vector of chromosome lengths.  In this case `xchr` should be either a
logical vector, of the same length as `L`, or a character string with
the name of the chromosome in `L` that correspond to the X
chromosome.

```{r ail_mult_chr}
ailped3 <- sim_ail_pedigree(ngen=12, npairs=30, nkids_per=3)
xodat3 <- sim_from_pedigree(ailped3, c("1"=100, "2"=75, "X"=100), "X")
xodat3alt <- sim_from_pedigree(ailped3, c("1"=100, "2"=75, "X"=100),
                               c(FALSE, FALSE, TRUE))
```


To indicate that all chromosomes are autosomes, you can use `xchr=""`,
`xchr=FALSE`, or `xchr=NULL`. For example:

```{r ail_mult_chr_no_X}
xodat3_noX <- sim_from_pedigree(ailped3, c("1"=100, "2"=75, "3"=100), "")
```

Having simulated multiple chromosomes with `sim_from_pedigree`, you
may wish to use `convert2geno` to convert those results to marker
genotypes. This is done by providing the output of `sim_from_pedigree`
as well as a genetic marker map that is a _list_ of vectors of marker
locations.

Let's first construct the marker map, assuming three chromosomes with
lengths 100, 75, and 100.

```{r construct_marker_map}
map <- list("1"=seq(0, 100, by=5),
            "2"=seq(0, 75, by=5),
            "X"=seq(0, 100, by=5))
for(i in seq(along=map))
    names(map[[i]]) <- paste0("m", names(map)[i], "_", map[[i]])
```

We then use `convert2geno`, ensuring that the inputs are both lists
with the same length.

```{r convert2geno_multchr}
geno <- convert2geno(xodat3, map)
```

For crosses like the DO, in which founder genotypes are needed, the
input `founder_geno` must be a list of matrices (one matrix per
chromosome).

Here are some simulated founder genotypes:

```{r do_founder_geno_mult_chr}
fg <- vector("list", length(map))
for(i in seq(along=map))
    fg[[i]] <- matrix(sample(1:2, 8*length(map[[i]]), replace=TRUE), nrow=8)
```

And now here is the simulation of a small DO population for multiple
chromosomes. When simulating the pedigree, we need to have 16 founders
(the 8 female founders and then the 8 male founders). After simulating
the genotypes with `sim_from_pedigree()`, we use
`collapse_do_alleles()` to collapse the alleles 9-16 (for the male
founders) into 1-8.


```{r sim_do_mult_chr}
doped <- sim_do_pedigree(ngen=4, nkids_per=1)
xodat <- sim_from_pedigree(doped, L=c("1"=100, "2"=75, "X"=100), "X")
xodat <- collapse_do_alleles(xodat)
geno <- convert2geno(xodat, map, fg)
```


## References

Broman KW (2012)
[Genotype probabilities at intermediate generations in the construction of recombinant inbred lines](https://doi.org/10.1534/genetics.111.132647).
Genetics 190:403-412
[doi: 10.1534/genetics.111.132647](https://doi.org/10.1534/genetics.111.132647)

Broman KW, Rowe LB, Churchill GA, Paigen K (2002)
[Crossover interference in the mouse](https://doi.org/10.1093/genetics/160.3.1123).
Genetics 160:1123-1131
[doi: 10.1093/genetics/160.3.1123](https://doi.org/10.1093/genetics/160.3.1123)

Copenhaver GP, Housworth EA, Stahl FW (2002)
[Crossover interference in arabidopsis](https://doi.org/10.1093/genetics/160.4.1631).
Genetics 160:1631-1639
[doi: 10.1093/genetics/160.4.1631](https://doi.org/10.1093/genetics/160.4.1631)

Foss E, Lande R, Stahl FW, Steinberg CM (1993)
[Chiasma interference as a function of genetic distance](https://doi.org/10.1093/genetics/92.2.573).
Genetics 133:681-691
[doi: 10.1093/genetics/92.2.573](https://doi.org/10.1093/genetics/92.2.573)

Kover PX, Valdar W, Trakalo J, Scarcelli N, Ehrenreich IM, Purugganan
MD, Durrant C, Mott R (2009)
[A Multiparent Advanced Generation Inter-Cross to fine-map quantitative traits in _Arabidopsis thaliana_](https://doi.org/10.1371/journal.pgen.1000551).
PLoS Genetics 5:e1000551
[doi: 10.1371/journal.pgen.1000551](https://doi.org/10.1371/journal.pgen.1000551)

McPeek MS, Speed TP (1995)
[Modeling interference in genetic recombination](https://doi.org/10.1093/genetics/139.2.1031).
Genetics 139:1031-1044
[doi: 10.1093/genetics/139.2.1031](https://doi.org/10.1093/genetics/139.2.1031)

Mott R, Flint J (2002)
[Simultaneous detection and fine mapping of quantitative trait loci in mice using heterogeneous stocks](https://doi.org/10.1093/genetics/160.4.1609).
Genetics 160:1609-1618
[doi: 10.1093/genetics/160.4.1609](https://doi.org/10.1093/genetics/160.4.1609)

Svenson KL, Gatti DM, Valdar W, Welsh CE, Cheng R, Chesler EJ, Palmer
AA, McMillan L, Churchill GA (2012)
[High-resolution genetic mapping using the mouse Diversity Outbred population.](https://doi.org/10.1534/genetics.111.132597).
Genetics 190:437-447
[doi: 10.1534/genetics.111.132597](https://doi.org/10.1534/genetics.111.132597)

Threadgill DW, Churchill GA (2012)
[Ten years of the Collaborative Cross](https://doi.org/10.1534/genetics.111.138032).
Genetics 190:291-294
[doi: 10.1534/genetics.111.138032](https://doi.org/10.1534/genetics.111.138032)

Zhao H, Speed TP, McPeek MS (1995)
[Statistical analysis of crossover interference using the chi-square model](https://doi.org/10.1534/genetics.111.138032).
Genetics 139:1045-1056
[doi: 10.1534/genetics.111.138032](https://doi.org/10.1534/genetics.111.138032)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_ril_pedigree.R
\name{sim_ril_pedigree}
\alias{sim_ril_pedigree}
\title{Generate a ril pedigree}
\usage{
sim_ril_pedigree(
  ngen = 20,
  selfing = FALSE,
  parents = 1:2,
  firstind = max(parents) + 1
)
}
\arguments{
\item{ngen}{Number of generations of inbreeding}

\item{selfing}{If TRUE, use selfing}

\item{parents}{Vector of the parents' IDs. Should be integers, and
length must be a power of 2 (i.e., 2, 4, 8, ...)}

\item{firstind}{Positive integer to assign to the first child. Must
be greater than \code{max(parents)}.}
}
\value{
A data frame with five columns: individual ID, mother ID,
father ID, sex, and generation.  Founders have \code{0} for mother
and father ID. Sex is coded 0 for female and 1 for male.
}
\description{
Generate a pedigree for multi-way recombinant inbred lines (a table
of individual, mom, dad, sex)
}
\examples{
tab <- sim_ril_pedigree(7)
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}}, \code{\link[=sim_do_pedigree]{sim_do_pedigree()}},
\code{\link[=sim_4way_pedigree]{sim_4way_pedigree()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_geno.R
\name{get_geno}
\alias{get_geno}
\title{Get genotype at a single position}
\usage{
get_geno(xodat, position)
}
\arguments{
\item{xodat}{The sort of detailed genotype/XO data generated by
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}}}

\item{position}{Position (in cM) for which to obtain genotypes}
}
\value{
A numeric matrix with two columns: the maternal and
paternal allele for each individual.
}
\description{
With data on the continuous crossover location information produced by
sim_from_pedigree, grab the genotype at a given position.
}
\examples{
# simulate AIL pedigree
tab <- sim_ail_pedigree(12, 30)
# simulate data from that pedigree
dat <- sim_from_pedigree(tab)
# get genotype at position 30 cM
geno <- get_geno(dat, 30)
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}}, \code{\link[=convert2geno]{convert2geno()}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_ail_pedigree_fix_n.R
\name{sim_ail_pedigree_fix_n}
\alias{sim_ail_pedigree_fix_n}
\title{Simulate AIL pedigree with fixed n}
\usage{
sim_ail_pedigree_fix_n(
  ngen = 12,
  nkids_per = 5,
  nsample_ngen = 150,
  npairs = NULL,
  method = c("last2", "sub2"),
  design = c("nosib", "random")
)
}
\arguments{
\item{ngen}{Number of generations of outbreeding}

\item{nkids_per}{Number of offspring per pair for the last
generation}

\item{nsample_ngen}{Number of individuals desired at the last
generation}

\item{npairs}{Number of breeding pairs at each generation. If
missing, we use 30 when \code{method="last2"} and 300 when
\code{method="sub2"}.}

\item{method}{Method used to generate pedigree: either expand at the last
two generations or generate a pedigree with a large number of pairs and
select a subset to have the desired sample size.}

\item{design}{How to choose crosses: either random but avoiding
siblings, or completely at random}
}
\value{
A data frame with five columns: individual ID, mother ID,
father ID, sex, and generation.  Founders have \code{0} for mother
and father ID. Sex is coded 0 for female and 1 for male.
}
\description{
Simulate a pedigree for advanced intercross lines (a table of
individual, mom, dad, sex) so that the last generation reaches a
desired sample size n
}
\details{
The default value for \code{npairs} depends on the choice of \code{method}.
For \code{method="last2"}, we use a default of \code{npairs=30}; for
\code{method="sub2"}, we use a default of \code{npairs=300}.
}
\examples{
tab <- sim_ail_pedigree_fix_n(12)
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}},
\code{\link[=sim_do_pedigree]{sim_do_pedigree()}}, \code{\link[=sim_4way_pedigree]{sim_4way_pedigree()}},
\code{\link[=sim_do_pedigree_fix_n]{sim_do_pedigree_fix_n()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_pedigree.R
\name{check_pedigree}
\alias{check_pedigree}
\title{Check a pedigree for errors}
\usage{
check_pedigree(pedigree, ignore_sex = FALSE)
}
\arguments{
\item{pedigree}{Numeric matrix or data frame with four columns: ID,
mom ID, dad ID, sex. Sex is coded as \code{0}=female,
\code{1}=male. There can be additional columns, but they'll be
ignored.}

\item{ignore_sex}{If TRUE, ignore the sex values completely
(appropriate for hermaphroditic species.)}
}
\value{
TRUE (invisibly) if everything is okay; otherwise gives an
error.
}
\description{
Perform a series of checks on the tabular data for a pedigree,
checking for problems
}
\details{
The parents should be listed before any of their
offspring. Founders should have 0's for mother and father; all
others should have non-zero values for the parents, and the parents
should appear in the pedigree. Father should be male and mothers
should be female (unless \code{ignore_sex=TRUE}). Individual
identifiers should be unique and non-zero. There should be no
missing values anywhere. (\code{NA}s are allowed in the sex column
if \code{ignore_sex=TRUE}.)
}
\examples{
tab <- sim_ril_pedigree(7)
check_pedigree(tab)

}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_ail_pedigree.R
\name{sim_ail_pedigree}
\alias{sim_ail_pedigree}
\title{Simulate AIL pedigree}
\usage{
sim_ail_pedigree(
  ngen = 12,
  npairs = 30,
  nkids_per = 5,
  design = c("nosib", "random")
)
}
\arguments{
\item{ngen}{Number of generations of outbreeding}

\item{npairs}{Number of breeding pairs at each generation}

\item{nkids_per}{Number of offspring per pair for the last
generation}

\item{design}{How to choose crosses: either random but avoiding
siblings, or completely at random}
}
\value{
A data frame with five columns: individual ID, mother ID,
father ID, sex, and generation.  Founders have \code{0} for mother
and father ID. Sex is coded 0 for female and 1 for male.
}
\description{
Simulate a pedigree for advanced intercross lines (a table of
individual, mom, dad, sex)
}
\details{
Advanced intercross lines (AIL) are generated from a pair of inbred lines.
We cross them and then cross the F1 to generate \code{npair} breeding pairs.
The subsequent \code{ngen} outbreeding generations then proceed by
crossing a male and female from the preceding generation (mated
completely at random, with \code{design="random"}, or avoiding
siblings, with \code{design="nosib"}). Each breeding pair gives a
single female and a single male to the next generation, except at
the last generation \code{nkids_per} offspring are mated, in equal
numbers male and female. (If \code{nkids_per} is an odd number, the
number of males and females in each sibship will differ by one,
alternating between sibships, with one additional female and then
one additional male.
}
\examples{
tab <- sim_ail_pedigree(12, 30)
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_do_pedigree]{sim_do_pedigree()}},
\code{\link[=sim_4way_pedigree]{sim_4way_pedigree()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/where_het.R
\name{where_het}
\alias{where_het}
\title{Find heterozygous regions}
\usage{
where_het(ind)
}
\arguments{
\item{ind}{An individual object, as output be
\code{\link[=create_parent]{create_parent()}} or \code{\link[=cross]{cross()}}}
}
\value{
A matrix with two columns; each row indicates the start and
end of a region where the individual is heterozygous
}
\description{
Find regions of heterozygosity in an individual
}
\examples{
mom <- create_parent(100, 1:2)
dad <- create_parent(100, 1:2)
child <- cross(mom, dad)
where_het(child)
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=convert2geno]{convert2geno()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_meiosis.R
\name{create_parent}
\alias{create_parent}
\title{Create a parent object}
\usage{
create_parent(L, allele = 1)
}
\arguments{
\item{L}{chromosome length in cM}

\item{allele}{vector of integers for alleles, of length 1 or 2}
}
\value{
A list with two components, for the individual's two
chromosomes.  Each is a list with alleles in chromosome intervals
(as integers) and locations of the right endpoints of those
intervals.
}
\description{
Create a parent object
}
\examples{
create_parent(100, 1)
create_parent(100, 1:2)
}
\seealso{
\code{\link[=cross]{cross()}}, \code{\link[=sim_meiosis]{sim_meiosis()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_4way_pedigree.R
\name{sim_4way_pedigree}
\alias{sim_4way_pedigree}
\title{Simulate pedigree for 4-way intercross}
\usage{
sim_4way_pedigree(ngen = 1, nsibs = 100)
}
\arguments{
\item{ngen}{Number of intercross generations (1 or 2)}

\item{nsibs}{Vector with number of siblings in the sibships in the
last generation.}
}
\value{
A data frame with five columns: individual ID, mother ID,
father ID, sex, and generation.  Founders have \code{0} for mother
and father ID. Sex is coded 0 for female and 1 for male.
}
\description{
Simulate a 4-way cross, among four inbred lines (a table of
individual, mom, dad, sex)
}
\details{
We start with a set of 4 individuals (representing four
inbred lines), and make a pair of crosses to generate a pair of
heterozygous individuals.  These are then crosses to generate a set
of F1 individuals. If \code{ngen==1}, we stop there, with
\code{sum(nsibs)} individuals in this last generation.  If
\code{gen==2}, we generate \code{length(nsibs)} male/female pairs
of F1 offspring; these are intercrossed to generate a set of
sibships, with lengths defined by the values in \code{nsibs}.
Individuals in the last generation are alternating female/male.
}
\examples{
# 100 F1s between heterozygous parents
tab <- sim_4way_pedigree(1, 100)
# could also do this
tab2 <- sim_4way_pedigree(1, rep(10, 10))

# 120 F2s in 10 sibships each of size 12
tab3 <- sim_4way_pedigree(ngen=2, rep(12, 10))
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_do_pedigree]{sim_do_pedigree()}},
\code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_meiosis.R
\name{cross}
\alias{cross}
\title{Cross two individuals}
\usage{
cross(
  mom,
  dad,
  m = 10,
  p = 0,
  xchr = FALSE,
  male = FALSE,
  obligate_chiasma = FALSE,
  Lstar = NULL
)
}
\arguments{
\item{mom}{An individual object, as produced by
\code{\link[=create_parent]{create_parent()}} or this function.}

\item{dad}{An individual object, as produced by
\code{\link[=create_parent]{create_parent()}} or this function.}

\item{m}{interference parameter for chi-square model}

\item{p}{proportion of crossovers coming from no-interference process}

\item{xchr}{If TRUE, simulate X chromosome}

\item{male}{If TRUE, simulate a male (matters only if
\code{xchr=TRUE})}

\item{obligate_chiasma}{If TRUE, require an obligate chiasma on the
4-strand bundle at meiosis.}

\item{Lstar}{Adjusted chromosome length, if
\code{obligate_chiasma=TRUE}. Calculated if not provided.}
}
\value{
A list with two components, for the individual's two
chromosomes.  Each is a list with alleles in chromosome intervals
(as integers) and locations of the right endpoints of those
intervals.
}
\description{
Simulate the cross of two individuals to create a
single progeny
}
\details{
Simulations are under the Stahl model with the
interference parameter being an integer. This is an extension of
the chi-square model, but with chiasmata being the superposition of
two processes, one following the chi-square model and the other
exhibiting no interference.
}
\examples{
mom <- create_parent(100, 1:2)
dad <- create_parent(100, 1:2)
child <- cross(mom, dad)
}
\seealso{
\code{\link[=create_parent]{create_parent()}}, \code{\link[=sim_meiosis]{sim_meiosis()}},
\code{\link[=sim_crossovers]{sim_crossovers()}}, \code{\link[=calc_Lstar]{calc_Lstar()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_meiosis.R
\name{sim_crossovers}
\alias{sim_crossovers}
\title{Simulate crossover locations using the Stahl model}
\usage{
sim_crossovers(L, m = 10, p = 0, obligate_chiasma = FALSE, Lstar = NULL)
}
\arguments{
\item{L}{length of chr in cM}

\item{m}{Interference parameter (\code{m=0} is no interference)}

\item{p}{Proportion of chiasmata from no-interference mechanism
(\code{p=0} gives pure chi-square model)}

\item{obligate_chiasma}{If TRUE, require an obligate chiasma on the
4-strand bundle at meiosis.}

\item{Lstar}{Adjusted chromosome length, if
\code{obligate_chiasma=TRUE}. Calculated if not provided.}
}
\value{
Numeric vector of crossover locations, in cM
}
\description{
Simulate crossover locations on a single meiotic product using the
Stahl model.
}
\details{
Chiasma locations are a superposition of two
processes: a proportion p exhibiting no interference, and a
proportion (1-p) following the chi-square model with interference
parameter m.  Crossover locations are derived by thinning the
chiasma locations with probability 1/2.

Simulations are under the Stahl model with the
interference parameter being an integer. This is an extension of
the chi-square model, but with chiasmata being the superposition of
two processes, one following the chi-square model and the other
exhibiting no interference.
}
\examples{
x <- sim_crossovers(200, 10, 0)
x <- sim_crossovers(200, 10, 0.04)
x <- sim_crossovers(100, 0, 0, obligate_chiasma=TRUE)

}
\references{
Copenhaver, G. P., Housworth, E. A. and Stahl, F. W. (2002) Crossover
interference in arabidopsis.  \emph{Genetics} \bold{160}, 1631--1639.

Foss, E., Lande, R., Stahl, F. W. and Steinberg, C. M. (1993) Chiasma
interference as a function of genetic distance. \emph{Genetics}
\bold{133}, 681--691.

Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis
of crossover interference using the chi-square model.  \emph{Genetics}
\bold{139}, 1045--1056.
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_cross.R
\name{CCcolors}
\alias{CCcolors}
\title{Collaborative Cross colors}
\usage{
CCcolors(palette = c("new", "original", "official"))
}
\arguments{
\item{palette}{Which version of the colors to use? (New or original)}
}
\value{
vector of eight colors
}
\description{
Get the vector of colors for the Collaborative Cross
}
\examples{
CCcolors()
}
\keyword{color}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collapse_do_alleles.R
\name{collapse_do_alleles}
\alias{collapse_do_alleles}
\title{Collapse alleles for simulated DO genotypes}
\usage{
collapse_do_alleles(xodata)
}
\arguments{
\item{xodata}{The sort of detailed genotype/crossover data
generated by \code{\link[=sim_from_pedigree]{sim_from_pedigree()}}.}
}
\value{
The input object, with alleles 9-16 replaced by 1-8.
}
\description{
When simulating Diversity Outbreds, we need to specify parents
1-16, with 9-16 being the males from strains 1-8.  This function
collapses replaces alleles 9-16 with 1-8, to make the result
ordinary DO-type data.
}
\examples{
# simulate DO pedigree
tab <- sim_do_pedigree(8)

# simulate genotypes for that pedigree
dat <- sim_from_pedigree(tab)
# collapse to alleles 1-8
dat <- collapse_do_alleles(dat)

# also works with data on multiple chromosomes
\donttest{dat <- sim_from_pedigree(tab, c("1"=100, "2"=75, "X"=100), xchr="X")
dat <- collapse_do_alleles(dat)}
}
\seealso{
\code{\link[=sim_do_pedigree]{sim_do_pedigree()}}, \code{\link[=sim_do_pedigree_fix_n]{sim_do_pedigree_fix_n()}},
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_do_pedigree_fix_n.R
\name{sim_do_pedigree_fix_n}
\alias{sim_do_pedigree_fix_n}
\title{Simulate a pedigree for Diversity Outbreds for a target sample size}
\usage{
sim_do_pedigree_fix_n(
  ngen = 12,
  nkids_per = 5,
  nccgen = 15,
  nsample_ngen = 150,
  npairs = NULL,
  method = c("last2", "sub2", "fixcc"),
  design = c("nosib", "random"),
  selc.method = c("byfamily", "byindiv")
)
}
\arguments{
\item{ngen}{Number of generations of outbreeding}

\item{nkids_per}{Number of offspring per pair for the last
generation}

\item{nccgen}{The number of generations for each CC line, only used
when \code{method} is not \code{"fixcc"}.}

\item{nsample_ngen}{Number of individuals desired at the last
generation}

\item{npairs}{Number of breeding pairs at each generation. If
missing, we use 30 when \code{method="last2"} and 300 when
\code{method="sub2"}.}

\item{method}{Method used to generate the pedigree: either expand
at the last two generations or generate a pedigree with a large
number of pairs and then select a subset to have the desired sample
size. With \code{method="fixcc"}, we use the pre-CC generations as performed at the
Jackson Lab.}

\item{design}{How to choose crosses: either random but avoiding
siblings, or completely at random}

\item{selc.method}{Method used to select the individuals from last
generation.}
}
\value{
A data frame with six columns: individual ID, mother ID, father
ID, sex, generation, and TRUE/FALSE indicator for whether DO or pre-DO.
Founders have \code{0} for mother and father ID. Sex is coded 0 for
female and 1 for male.
}
\description{
Simulate a pedigree for Diversity Outbred (DO) mice (a table of
individual, mom, dad, sex) so that the last generation reaches a
desired sample size.
}
\details{
The default number of breeding pairs depends on the chosen
\code{method}. With \code{method="last2"}, the default is \code{npairs=30};
with \code{method="sub2"}, the default is \code{npairs=300};
with \code{method="fixcc"}, \code{npairs} is ignored and is fixed at 144.
}
\examples{
tab <- sim_do_pedigree_fix_n(8)
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}},
\code{\link[=sim_do_pedigree]{sim_do_pedigree()}}, \code{\link[=sim_4way_pedigree]{sim_4way_pedigree()}},
\code{\link[=sim_ail_pedigree_fix_n]{sim_ail_pedigree_fix_n()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_meiosis.R
\name{calc_Lstar}
\alias{calc_Lstar}
\title{Calculate adjusted chromosome length for obligate chiasma}
\usage{
calc_Lstar(L, m = 0, p = 0)
}
\arguments{
\item{L}{Length of chromosome (in cM); must be > 50}

\item{m}{Interference parameter for chi-square model}

\item{p}{Proportion of chiasmata coming from no-interference
process}
}
\value{
Adjusted length of chromosome
}
\description{
Calculate the reduced chromosome length that will give the target
expected number of chiasmata when conditioning on there being at
least one chiasma on the four-strand bundle.
}
\examples{
calc_Lstar(100, 0, 0)
calc_Lstar(60, 10, 0.1)
}
\seealso{
\code{\link[=cross]{cross()}}, \code{\link[=sim_meiosis]{sim_meiosis()}},
\code{\link[=sim_crossovers]{sim_crossovers()}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_do_pedigree.R
\name{sim_do_pedigree}
\alias{sim_do_pedigree}
\title{Simulate a pedigree for Diversity Outbred mice}
\usage{
sim_do_pedigree(
  ngen = 12,
  npairs = 144,
  ccgen = rep(4:12, c(21, 64, 24, 10, 5, 9, 5, 3, 3)),
  nkids_per = 5,
  design = c("nosib", "random")
)
}
\arguments{
\item{ngen}{Number of generations of outbreeding}

\item{npairs}{Number of breeding pairs at each generation}

\item{ccgen}{Vector of length \code{npairs}, with the number of
generations for each CC line. If length 1, it is repeated to a
vector of length \code{npairs}.}

\item{nkids_per}{Number of offspring per pair for the last
generation}

\item{design}{How to choose crosses: either random but avoiding
siblings, or completely at random}
}
\value{
A data frame with six columns: individual ID, mother ID, father
ID, sex, generation, and TRUE/FALSE indicator for whether DO or pre-DO.
Founders have \code{0} for mother and father ID. Sex is coded 0 for
female and 1 for male.
}
\description{
Simulate a pedigree for generating Diversity Outbred (DO) mice (a
table of individual, mom, dad, sex).
}
\details{
Diversity outbred (DO) mice are generated from a set of 8 inbred
lines. We need two individuals from each line (one female and one
male) as the order of the initial crosses will be randomized; for
example, sometimes the individual from line 1 will be a mother and
sometimes a father. The founders are numbered 1-8 for the females
from the 8 lines, and 9-16 for the corresponding males.

Diversity Outbred mice are generated by first creating a panel of
partially-inbred 8-way RIL (the so-called pre-CC, for
pre-Collaborative Cross). The \code{ccgen} argument specifies the
number of inbreeding generations for each of the CC lines. We
generate a pre-CC line for each of the \code{npairs} breeding
pairs, and generate a sibling pair from each as the starting
material.

The subsequent \code{ngen} outbreeding generations then proceed by
crossing a male and female from the preceding generation (mated
completely at random, with \code{design="random"}, or avoiding
siblings, with \code{design="nosib"}). Each breeding pair gives a
single female and a single male to the next generation, except at
the last generation \code{nkids_per} offspring are mated, in equal
numbers male and female. (If \code{nkids_per} is an odd number, the
number of males and females in each sibship will differ by one,
alternating between sibships, with one additional female and then
one additional male.

The default for \code{ccgen} is taken from Figure 1 of Svenson et
al. (2012).
}
\examples{
tab <- sim_do_pedigree(8)
}
\references{
Svenson KL, Gatti DM, Valdar W, Welsh CE, Cheng R, Chesler EJ,
Palmer AA, McMillan L, Churchill GA (2012) High-resolution genetic
mapping using the mouse Diversity Outbred population. Genetics
190:437-447
}
\seealso{
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}},
\code{\link[=sim_4way_pedigree]{sim_4way_pedigree()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_cross.R
\name{plot_ind}
\alias{plot_ind}
\title{Plot an individual}
\usage{
plot_ind(
  ind,
  center,
  chrlength = 30,
  chrwidth = 3,
  gap = 3,
  col = CCcolors(),
  border = "black",
  lend = 1,
  ljoin = 1,
  allborders = FALSE,
  ...
)
}
\arguments{
\item{ind}{An individual object, as output by
\code{\link[=create_parent]{create_parent()}} or \code{\link[=cross]{cross()}}}

\item{center}{(x,y) vector for the center of the individual}

\item{chrlength}{Length of chromosomes (Can be a vector of length
2, in which case the two chromosomes will be different lengths,
aligned at the top. This is for the X chromosome.)}

\item{chrwidth}{Width of chromosomes}

\item{gap}{Gap between chromosomes}

\item{col}{Vector of colors}

\item{border}{Color for border}

\item{lend}{Passed to \code{\link[graphics:rect]{graphics::rect()}}}

\item{ljoin}{Passed to \code{\link[graphics:rect]{graphics::rect()}}}

\item{allborders}{If TRUE, put borders around all segments}

\item{...}{Additional arguments passed to rect()}
}
\value{
None.
}
\description{
Add an individual, as a pair of chromosomes, to a plot
}
\examples{
\dontshow{set.seed(67452378)}
mom <- create_parent(100, 1:2)
dad <- create_parent(100, 3:4)
kid <- cross(mom, dad)
plot(0,0, type="n", xlim=c(0, 100), ylim=c(0,100),
     xaxt="n", yaxt="n", xlab="", ylab="")
loc <- list(c(25,75), c(75,75), c(50,25))
plot_ind(mom, loc[[1]])
plot_ind(dad, loc[[2]])
plot_ind(kid, loc[[3]])
plot_crosslines(loc[[1]], loc[[2]], loc[[3]])
}
\seealso{
\code{\link[=plot_crosslines]{plot_crosslines()}}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_from_pedigree.R
\name{sim_from_pedigree}
\alias{sim_from_pedigree}
\title{Simulate genotypes for pedigree}
\usage{
sim_from_pedigree(
  pedigree,
  L = 100,
  xchr = FALSE,
  m = 10,
  p = 0,
  obligate_chiasma = FALSE
)
}
\arguments{
\item{pedigree}{Matrix or data frame describing a pedigree, with first four
columns being individual ID, mom ID, dad ID, and sex (female as
\code{0}, male as \code{1}).}

\item{L}{Length of chromosome in cM (or a vector of chromosome lengths)}

\item{xchr}{If TRUE, simulate X chromosome. (If \code{L} is a
vector, this should be a vector of TRUE/FALSE values, of the same
length as \code{L}, or a character string with the name of the X
chromosome, in \code{L}.)}

\item{m}{Crossover interference parameter, for chi-square model
(m=0 corresponds to no interference).}

\item{p}{proportion of crossovers coming from no-interference process}

\item{obligate_chiasma}{If TRUE, require an obligate chiasma on the
4-strand bundle at meiosis.}
}
\value{
A list with each component being the data for one
individual, as produced by the \code{\link[=cross]{cross()}} function.  Those
results are a list with two components, corresponding to the
maternal and paternal chromosomes. The chromosomes are represented
as lists with two components: an integer vector of alleles in
chromosome intervals, and a numeric vector of locations of the
right-endpoints of those intervals; these two vectors should have
the same length.

If the input \code{L} is a vector, in order to simulate multiple
chromosomes at once, then the output will be a list with length
\code{length(L)}, each component being a chromosome and having the
form described above.
}
\description{
Simulate genotypes along one chromosome for a pedigree
}
\examples{
# simulate AIL pedigree
tab <- sim_ail_pedigree(12, 30)
# simulate data from that pedigree
dat <- sim_from_pedigree(tab)
# simulate multiple chromosomes
dat <- sim_from_pedigree(tab, c("1"=100, "2"=75, "X"=100), xchr="X")
}
\seealso{
\code{\link[=check_pedigree]{check_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}},
\code{\link[=sim_from_pedigree_allchr]{sim_from_pedigree_allchr()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mouseL_mgi-data.R
\docType{data}
\name{mouseL_mgi}
\alias{mouseL_mgi}
\title{Mouse chromosome lengths}
\format{
A numeric vector with lengths in cM for the 20 mouse chromosomes.
}
\source{
Taken from Table 1 of Cox et al. (2009) A new standard
genetic map for the laboratory mouse. Genetics 182:1335-1344.
\doi{10.1534/genetics.109.105486}
}
\usage{
data(mouseL_mgi)
}
\description{
Mouse chromosome lengths in cM from the Mouse Genome Informatics
(MGI) standard map.
}
\examples{
data(mouseL_mgi)
}
\seealso{
mouseL_cox
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert2geno.R
\name{convert2geno}
\alias{convert2geno}
\title{Convert continuous allele information into marker genotypes}
\usage{
convert2geno(xodat, map, founder_geno = NULL, shift_map = FALSE)
}
\arguments{
\item{xodat}{The sort of detailed genotype/crossover data generated by
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}}}

\item{map}{vector of marker locations; can also be a list of such
vectors (one per chromosome), in which case xodat and founder_geno
must be lists with the same length.}

\item{founder_geno}{Optional matrix (size \code{n_founders} x
\code{length(map)}) of founder genotypes. If coded as 1/2 (or 1/3),
results are 1/2/3 genotypes. If coded as A/T/G/C/N, results are
A/T/G/C/N/H genotypes. If coded as letters A-H for the 8 founders,
results are two-letter genotypes AA-HH with 36 possible values.}

\item{shift_map}{If TRUE, shift genetic map to start at 0}
}
\value{
If \code{founder_geno} is provided or there are just two
founders, the result is a numeric matrix of genotypes, individuals
x markers, with genotypes 1/2/3 codes for 11/12/22 genotypes.

If \code{founder_geno} is not provided and there are more than two
founders, the result is a 3-dimensional array, individuals x
markers x alleles, with the third dimensional corresponding to the
maternal and paternal allele.

If the input \code{map} is a list (the components being
chromosomes), then \code{xodat} and \code{founder_geno} must be
lists of the same length, and the result will be a list of
matrices.
}
\description{
Convert the continuous crossover location information produced by
sim_from_pedigree to marker genotypes
}
\examples{
# simulate AIL pedigree
tab <- sim_ail_pedigree(12, 30)
# simulate data from that pedigree
dat <- sim_from_pedigree(tab)
# marker map (could also use sim.map in R/qtl)
map <- seq(0, 100, by=5)
names(map) <- paste0("marker", seq(along=map))
# convert data to marker genotypes
geno <- convert2geno(dat, map)


# AIL with multiple chromosomes
dat <- sim_from_pedigree(tab, c("1"=100, "2"=75, "X"=100), xchr="X")
# marker map
multmap <- list("1"=seq(0, 100, by=5),
                "2"=seq(0, 75, by=5),
                "X"=seq(0, 100, by=5))
for(i in 1:3)
  names(multmap[[i]]) <- paste0("marker", i, "_", 1:length(map[[i]]))
geno <- convert2geno(dat, multmap)

# simulate DO pedigree
tab <- sim_do_pedigree(8)
# simulate data from that pedigree
dat <- sim_from_pedigree(tab)
# simulate founder snp alleles
fg <- matrix(sample(1:2, 8*length(map), repl=TRUE), nrow=8)
# for DO, need female & male founders (to deal with X chr)
fg <- rbind(fg, fg)
# convert dat to SNP genotypes
geno <- convert2geno(dat, map, fg)
# if fg not provided, result is a 3d array
genoarray <- convert2geno(dat, map)
}
\seealso{
\code{\link[=get_geno]{get_geno()}}, \code{\link[=sim_from_pedigree]{sim_from_pedigree()}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert2geno_allchr.R
\name{convert2geno_allchr}
\alias{convert2geno_allchr}
\title{Convert continuous allele information into marker genotypes for
multiple chromosomes}
\usage{
convert2geno_allchr(
  xodat,
  map,
  id = NULL,
  founder_geno = NULL,
  return.matrix = TRUE,
  shift_map = FALSE
)
}
\arguments{
\item{xodat}{The sort of detailed genotype/crossover data generated
by \code{\link[=sim_from_pedigree_allchr]{sim_from_pedigree_allchr()}}}

\item{map}{marker locations, a list with elements for each
chromosome}

\item{id}{ids for which individuals genotypes is desired}

\item{founder_geno}{Optional list of matrices (one per chromosome)
of size \code{n_founders} x \code{n_markers}, with the founder genotypes.
If coded as 1/2 (or 1/3), results are 1/2/3
genotypes. If coded as A/T/G/C/N, results are A/T/G/C/N/H
genotypes. If coded as letters A-H (in the case of 8 founders), results are
two-letter genotypes AA-HH with 36 possible values.}

\item{return.matrix}{If FALSE, the result is a list of length
\code{n_chrs}, otherwise it is converted into a matrix if size
\code{length(id)} x \code{n_markers}.}

\item{shift_map}{If TRUE, shift genetic map to start at 0}
}
\value{
If \code{founder_geno} is provided or there are just two
founders, the result is a numeric matrix of genotypes, individuals
x markers, with genotypes 1/2/3 codes for 11/12/22 genotypes. If
there are more than two founders and \code{founder_geno} are
letters, the result is a character matrix, too.

If \code{founder_geno} is not provided and there are more than two
founders, the result is a 3-dimensional array, individuals x
markers x alleles, with the third dimensional corresponding to the
maternal and paternal allele.
}
\description{
Wrap up of convert2geno to adequate multiple chromosomes.
}
\examples{
library(qtl)
# marker map
map <- sim.map(len=rep(100, 19), n.mar=10, include.x=FALSE)
# simulate AIL pedigree
tab <- sim_ail_pedigree(12, 30)
# simulate data from that pedigree
dat <- sim_from_pedigree_allchr(tab, map)
names(map) <- paste0("marker", seq(along=map))
# convert data to marker genotypes
id <- which(tab[, "gen"]==12)
geno <- convert2geno_allchr(dat, map, id)

}
\seealso{
\code{\link[=convert2geno]{convert2geno()}}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_cross.R
\name{plot_crosslines}
\alias{plot_crosslines}
\title{Plot cross lines}
\usage{
plot_crosslines(
  momloc,
  dadloc,
  kidsloc,
  gap = 3,
  chrlength = 30,
  cex = 1.5,
  lwd = 2,
  arrow_length = 0.1,
  col = "black",
  ...
)
}
\arguments{
\item{momloc}{An (x,y) vector with center location for mother}

\item{dadloc}{An (x,y) vector with center location for mother}

\item{kidsloc}{Either an (x,y) vector with center location for a
kid, or a list of such for multiple kids}

\item{gap}{Gap arrows and points/rectangles}

\item{chrlength}{Length of chromosomes}

\item{cex}{Character expansion for x point}

\item{lwd}{Line width for points, segments, and arrows}

\item{arrow_length}{The \code{length} parameter in the call to
\code{\link[graphics:arrows]{graphics::arrows()}}}

\item{col}{Color of lines and points}

\item{...}{Additional arguments passed to arrows() and segments()}
}
\value{
None.
}
\description{
Add lines for a cross
}
\examples{
\dontshow{set.seed(67452378)}
mom <- create_parent(100, 1:2)
dad <- create_parent(100, 3:4)
kids <- lapply(1:4, function(junk) cross(mom, dad))
plot(0,0, type="n", xlim=c(0, 100), ylim=c(0,100),
     xaxt="n", yaxt="n", xlab="", ylab="")
loc <- list(c(25,75), c(75,75), c(12.5,25), c(37.5,25), c(62.5, 25), c(87.5,25))
plot_ind(mom, loc[[1]])
plot_ind(dad, loc[[2]])
for(i in 1:4) plot_ind(kids[[i]], loc[[i+2]])
plot_crosslines(loc[[1]], loc[[2]], loc[3:6])
}
\seealso{
\code{\link[=plot_ind]{plot_ind()}}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AILped-data.R
\docType{data}
\name{AILped}
\alias{AILped}
\title{Example AIL pedigree}
\format{
A data frame with five columns: individual id, mom, dad, sex (0
for females and 1 for males) and generation.
}
\source{
Derived from the pedF8 dataset in the QTLRel package,
\url{https://cran.r-project.org/package=QTLRel}
}
\usage{
data(AILped)
}
\description{
Example matrix describing the pedigree for advanced intercross lines
}
\examples{
data(AILped)
x <- sim_from_pedigree(AILped)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_from_pedigree_allchr.R
\name{sim_from_pedigree_allchr}
\alias{sim_from_pedigree_allchr}
\title{Simulate genotypes for pedigree for multiple chromosomes}
\usage{
sim_from_pedigree_allchr(
  pedigree,
  map,
  m = 10,
  p = 0,
  obligate_chiasma = FALSE
)
}
\arguments{
\item{pedigree}{Matrix or data frame describing a pedigree, with first four
columns being individual ID, mom ID, dad ID, and sex (female as
\code{0}, male as \code{1}).}

\item{map}{marker locations, a list with elements for each
chromosome}

\item{m}{Crossover interference parameter, for chi-square model
(m=0 corresponds to no interference).}

\item{p}{proportion of crossovers coming from no-interference process}

\item{obligate_chiasma}{If TRUE, require an obligate chiasma on the
4-strand bundle at meiosis.}
}
\value{
A list with each component being the result from
\code{sim_from_pedigree}, of length same as \code{map}.
}
\description{
Simulate genotypes along all chromosomes for a pedigree. This is a
wrap up of sim_from_pedigree.
}
\examples{
library(qtl)
# marker map
map <- sim.map(len=rep(100, 19), n.mar=10, include.x=FALSE)
# simulate AIL pedigree
tab <- sim_ail_pedigree(12, 30)
# simulate data from that pedigree
dat <- sim_from_pedigree_allchr(tab, map)

}
\seealso{
\code{\link[=check_pedigree]{check_pedigree()}},
\code{\link[=sim_ril_pedigree]{sim_ril_pedigree()}}, \code{\link[=sim_ail_pedigree]{sim_ail_pedigree()}}
\code{\link[=sim_from_pedigree]{sim_from_pedigree()}}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_meiosis.R
\name{sim_meiosis}
\alias{sim_meiosis}
\title{Simulate meiosis}
\usage{
sim_meiosis(parent, m = 10, p = 0, obligate_chiasma = FALSE, Lstar = NULL)
}
\arguments{
\item{parent}{An individual object, as output by
\code{\link[=create_parent]{create_parent()}} or \code{\link[=cross]{cross()}}}

\item{m}{interference parameter for chi-square model}

\item{p}{Proportion of chiasmata coming from no-interference process.}

\item{obligate_chiasma}{If TRUE, require an obligate chiasma on the
4-strand bundle at meiosis.}

\item{Lstar}{Adjusted chromosome length, if
\code{obligate_chiasma=TRUE}. Calculated if not provided.}
}
\value{
A list with alleles in chromosome intervals (as integers)
and locations of the right endpoints of those intervals.
}
\description{
Output a random meiotic product from an input individual.
}
\details{
Simulations are under the Stahl model with the
interference parameter being an integer. This is an extension of
the chi-square model, but with chiasmata being the superposition of
two processes, one following the chi-square model and the other
exhibiting no interference.
}
\examples{
ind <- create_parent(100, 1:2)
prod <- sim_meiosis(ind)
}
\references{
Copenhaver, G. P., Housworth, E. A. and Stahl, F. W. (2002) Crossover
interference in arabidopsis.  \emph{Genetics} \bold{160}, 1631--1639.

Foss, E., Lande, R., Stahl, F. W. and Steinberg, C. M. (1993) Chiasma
interference as a function of genetic distance. \emph{Genetics}
\bold{133}, 681--691.

Zhao, H., Speed, T. P. and McPeek, M. S. (1995) Statistical analysis
of crossover interference using the chi-square model.  \emph{Genetics}
\bold{139}, 1045--1056.
}
\seealso{
\code{\link[=create_parent]{create_parent()}}, \code{\link[=cross]{cross()}},
\code{\link[=sim_crossovers]{sim_crossovers()}}, \code{\link[=calc_Lstar]{calc_Lstar()}}
}
\keyword{datagen}
